@@ -18,7 +18,7 @@
  *   with this program; if not, write to the Free Software Foundation, Inc.,
  *   51 Franklin St, Fifth Floor, Boston, MA  02110-1301 USA.
  *
- *   $Id: libummod.h 298 2007-02-15 14:45:02Z garden $
+ *   $Id: libummod.h 649 2009-04-23 05:44:01Z rd235 $
  *
  * 
  */
@@ -27,8 +27,8 @@
 #include <unistd.h>
 #include <linux/types.h>
 #include <sys/types.h>
-#include <linux/dirent.h>
 #include <linux/unistd.h>
+#include <linux_dirent.h>
 #include <errno.h>
 
 int getdents(unsigned int fd, struct dirent *dirp, unsigned int count);
@@ -114,6 +114,14 @@ int _llseek(unsigned int fd, unsigned long offset_high,
 #undef __NR_setfsuid
 #define __NR_setfsuid __NR_setfsuid32
 #endif
+#ifdef __NR_getegid32
+#undef __NR_getegid
+#define __NR_getegid __NR_getegid32
+#endif
+#ifdef __NR_setfsgid32
+#undef __NR_setfsgid
+#define __NR_setfsgid __NR_setfsgid32
+#endif
 
 /*#if defined(__x86_64__)
 #define __NR__newselect __NR_select
