@@ -9,6 +9,7 @@
 #ifndef _UMNET_H_
 #define _UMNET_H_
 #include <stdint.h>
+#include <module.h>
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <sys/socket.h>
@@ -54,6 +55,8 @@ struct umnet_operations {
 	ssize_t (*recv) (int, const void *, size_t, int);
 	ssize_t (*sendto) (int, const void *, size_t, int, const struct sockaddr *, socklen_t);
 	ssize_t (*recvfrom) (int, void *, size_t, int, struct sockaddr *, socklen_t *);
+	ssize_t (*recvmsg)(int, struct msghdr *, int);
+	ssize_t (*sendmsg)(int, const struct msghdr *, int);
 	int (*setsockopt) (int, int, int, const void *, socklen_t);
 	int (*getsockopt) (int, int, int, void *, socklen_t *);
 	ssize_t (*read) (int, void *, size_t);
