@@ -19,18 +19,17 @@
  *   with this program; if not, write to the Free Software Foundation, Inc.,
  *   51 Franklin St, Fifth Floor, Boston, MA  02110-1301 USA.
  *
- *   $Id: libummod.c 282 2007-01-18 12:33:03Z garden $
+ *   $Id: libummod.c 650 2009-04-23 11:22:50Z rd235 $
  *
  */
 #include <unistd.h>
 #include <linux/types.h>
 #include <sys/types.h>
-#include <linux/dirent.h>
+#include <linux_dirent.h>
 #include <linux/unistd.h>
 #include <errno.h>
 #include <config.h>
 
-
 int getdents(unsigned int fd, struct dirent *dirp, unsigned int count)
 {
 	return syscall(__NR_getdents, fd, dirp, count);
