@@ -18,7 +18,7 @@
  *   with this program; if not, write to the Free Software Foundation, Inc.,
  *   51 Franklin St, Fifth Floor, Boston, MA  02110-1301 USA.
  *
- *   $Id: um_lib.c 669 2009-05-30 16:24:08Z rd235 $
+ *   $Id: um_lib.c 866 2010-08-11 07:17:33Z rd235 $
  *
  */   
 #include <sys/syscall.h>
@@ -27,6 +27,7 @@
 #include <config.h>
 #include <um_lib.h>
 
+#ifdef OLDVIRSC
 static long int_virnsyscall(long virscno,int n,long arg1,long arg2,long arg3,long arg4,long arg5,long arg6) {
 	struct __sysctl_args scarg;
 	long args[6]={arg1,arg2,arg3,arg4,arg5,arg6};
@@ -38,22 +39,30 @@ static long int_virnsyscall(long virscno,int n,long arg1,long arg2,long arg3,lon
 	scarg.newlen=n;
 	return syscall(__NR__sysctl,&scarg);
 }
+#else
+static long int_virnsyscall(long virscno,int n,long arg1,long arg2,long arg3,long arg4,long arg5,long arg6) {
+	long args[6]={arg1,arg2,arg3,arg4,arg5,arg6};
+	return syscall(__NR_pivot_root,NULL,n,virscno,args);
+}
+#endif
 
 long (*virnsyscall)() = int_virnsyscall;
 
-int um_add_service(int position,char *path)
+int um_check_viewos(void)
 {
-	return virsyscall3(VIRUMSERVICE,ADD_SERVICE,position,path);
+	struct viewinfo info;
+	int rv=um_view_getinfo(&info);
+	return (rv==0);
 }
 
-int um_del_service(int code)
+int um_add_service(char *path,int permanent)
 {
-	return virsyscall2(VIRUMSERVICE,DEL_SERVICE,code);
+	return virsyscall3(VIRUMSERVICE,ADD_SERVICE,path,permanent);
 }
 
-int um_mov_service(int code, int position)
+int um_del_service(char *name)
 {
-	return virsyscall3(VIRUMSERVICE,MOV_SERVICE,code,position);
+	return virsyscall2(VIRUMSERVICE,DEL_SERVICE,name);
 }
 
 int um_list_service(char *buf, int len)
@@ -61,37 +70,32 @@ int um_list_service(char *buf, int len)
 	return virsyscall3(VIRUMSERVICE,LIST_SERVICE,buf,len);
 }
 
-int um_name_service(int code, char *buf, int len)
+int um_name_service(char *name, char *buf, int len)
 {
-	return virsyscall4(VIRUMSERVICE,NAME_SERVICE,code,buf,len);
-}
-
-int um_lock_service(int invisible)
-{
-	return virsyscall2(VIRUMSERVICE,LOCK_SERVICE,invisible);
+	return virsyscall4(VIRUMSERVICE,NAME_SERVICE,name,buf,len);
 }
 
 int um_view_getinfo(struct viewinfo *info)
 {
-	return virsyscall2(VIRUMSERVICE,UMVIEW_GETINFO,info);
+	return virsyscall2(VIRUMSERVICE,VIEWOS_GETINFO,info);
 }
 
 int um_setviewname(char *name)
 {
-	return virsyscall2(VIRUMSERVICE,UMVIEW_SETVIEWNAME,name);
+	return virsyscall2(VIRUMSERVICE,VIEWOS_SETVIEWNAME,name);
 }
 
 int um_killall(int signo)
 {
-	return virsyscall2(VIRUMSERVICE,UMVIEW_KILLALL,signo);
+	return virsyscall2(VIRUMSERVICE,VIEWOS_KILLALL,signo);
 }
 
 int um_attach(int pid)
 {
-	return virsyscall2(VIRUMSERVICE,UMVIEW_ATTACH,pid);
+	return virsyscall2(VIRUMSERVICE,VIEWOS_ATTACH,pid);
 }
 
 int um_fsalias(char *alias,char *filesystemname)
 {
-	return virsyscall3(VIRUMSERVICE,UMVIEW_FSALIAS,alias,filesystemname);
+	  return virsyscall3(VIRUMSERVICE,VIEWOS_FSALIAS,alias,filesystemname);
 }
