@@ -18,7 +18,7 @@
  *   with this program; if not, write to the Free Software Foundation, Inc.,
  *   51 Franklin St, Fifth Floor, Boston, MA  02110-1301 USA.
  *
- *   $Id: testmodul2.c 282 2007-01-18 12:33:03Z garden $
+ *   $Id: testmodul2.c 1003 2011-09-06 10:32:34Z rd235 $
  *
  */
 #include <stdio.h>
@@ -26,30 +26,24 @@
 #include "module.h"
 #include "gdebug.h"
 
-struct service s;
-
-static epoch_t alwaysfalse()
-{
-	return 0;
-}
+static struct service s;
+VIEWOS_SERVICE(s)
 
 static void
 __attribute__ ((constructor))
 init (void)
 {
-	GMESSAGE("testmodul2 init");
-	s.name="Test Module 2";
-	s.code=0xfd;
-	s.checkfun=alwaysfalse;
+	printk(KERN_NOTICE "testmodul2 init\n");
+	s.name="test2";
+	s.description="Test Module 2";
 	s.syscall=NULL;
 	s.socket=NULL;
-	add_service(&s);
 }
 
 static void
 __attribute__ ((destructor))
 fini (void)
 {
-	GMESSAGE("testmodul2 fini");
+	printk(KERN_NOTICE "testmodul2 fini\n");
 }       
 
