@@ -31,6 +31,7 @@
 #define VIEWFS_MINCOW 0x10
 #define VIEWFS_RENEW 0x100 
 #define VIEWFS_WOK   0x200 
+#define VIEWFS_VSTAT 0x400 
 
-int viewfsarg(char *opts,int *pflags,char ***pexceptions);
+int viewfsargs(char *opts,int *pflags,char ***pexceptions);
 #endif
