@@ -17,7 +17,7 @@
  *   with this program; if not, write to the Free Software Foundation, Inc.,
  *   51 Franklin St, Fifth Floor, Boston, MA  02110-1301 USA.
  *
- *   $Id: umbinfmt.c 648 2009-04-22 22:50:01Z rd235 $
+ *   $Id: umbinfmt.c 971 2011-08-03 14:38:28Z rd235 $
  *
  */   
 #include <stdio.h>
@@ -43,8 +43,6 @@
 #include "libummod.h"
 //#include "umbinfmt.h"
 
-#define UMBINFMT_SERVICE_CODE 0x03
-
 /* Enable umbinfmt own debug output */
 
 //#define __UMBINFMT_DEBUG__ 1   /* it is better to enable it from makefile */
@@ -64,6 +62,7 @@
 #define syscall_getgid getgid
 
 static struct service s;
+VIEWOS_SERVICE(s)
 
 struct binfileinfo {
 	struct umregister *reg;
@@ -98,24 +97,13 @@ static char statusfile[]="status";
 
 struct umbinfmt {
 	char *path;
-	struct timestamp tst;
 	unsigned char enabled;
 	char flags;
 	int inuse;
 	struct umregister *head;
+	struct ht_elem *binfmt_ht;
 };
 
-#define MNTTABSTEP 4 /* must be a power of two */
-#define MNTTABSTEP_1 (MNTTABSTEP-1)
-#define FILETABSTEP 4 /* must be a power of two */
-#define FILETABSTEP_1 (FILETABSTEP-1)
-
-static struct binfileinfo **filetab=NULL;
-static int filetabmax=0;
-
-static struct umbinfmt **mnttab=NULL;
-static int mnttabmax=0;
-
 struct umbinfmt_dirent64 {
 	__ino64_t             d_ino;
 	__off64_t             d_off;
@@ -124,16 +112,6 @@ struct umbinfmt_dirent64 {
 	char            d_name[0];
 };
 
-#if 0
-struct umbinfmt_dirent {
-	__ino_t            d_ino;
-	__off_t            d_off;
-	unsigned short  d_reclen;
-	//unsigned char   d_type; /* this field does not exist in getdents provided data */
-	char            d_name[0];
-};
-#endif
-
 #ifdef __UMBINFMT_DEBUG__
 static void printdebug(int level, const char *file, const int line, const char *func, const char *fmt, ...) {
 	va_list ap;
@@ -141,172 +119,53 @@ static void printdebug(int level, const char *file, const int line, const char *
 	if (level >= __UMBINFMT_DEBUG_LEVEL__) {
 		va_start(ap, fmt);
 #ifdef _PTHREAD_H
-		fprint2("[%d:%lu] dev %s:%d %s(): ", getpid(), pthread_self(), file, line, func);
+		printk("[%d:%lu] dev %s:%d %s(): ", getpid(), pthread_self(), file, line, func);
 #else
-		fprint2("[%d] dev %s:%d %s(): ", getpid(), file, line, func);
+		printk("[%d] dev %s:%d %s(): ", getpid(), file, line, func);
 #endif
-		vfprint2(fmt, ap);
-		fprint2("\n");
+		vprintk(fmt, ap);
+		printk("\n");
 		va_end(ap);
 	}
 }
 #endif
 
-/* search a binfmt, returns the context i.e. the index of info for mounted file
- * -1 otherwise */
-static struct umbinfmt *searchbmfile(char *path)
+static int searchbinfmt(struct umbinfmt *fc,struct binfmt_req *req)
 {
-	register int i;
-	struct umbinfmt *result=NULL;
-	epoch_t maxepoch=0;
-	int maxi=-1;
-	PRINTDEBUG(0,"SearchContext:%s\n",path);
-	for (i=0;i<mnttabmax;i++)
-	{
-		epoch_t e;
-		if ((mnttab[i] != NULL)) {
-			if (strncmp(path,mnttab[i]->path,strlen(mnttab[i]->path)) == 0 &&
-					((e=tst_matchingepoch(&(mnttab[i]->tst))) > maxepoch)) {
-				maxi=i;
-				maxepoch=e;
-			}
-		}
-	}
-	if (maxi >= 0)
-		result=mnttab[maxi];
-	return result;
-}
-
-/* search for a match with an existing entry and returns the interpreter */
-
-static epoch_t searchbinfmt(struct binfmt_req *req)
-{
-	register int ifc;
-	char buf[128];
-	memset(buf,0,128);
-	int fd;
-	fd=open(req->path,O_RDONLY);
-	if (fd >= 0) {
-		read(fd, buf, 128);
-		close(fd);
-	}
-	req->interp=NULL;
-	for (ifc=0;ifc<mnttabmax;ifc++)
-		if (mnttab[ifc] != NULL) {
-			struct umbinfmt *fc=mnttab[ifc];
-			if (fc->enabled) {
-				struct umregister *scan=fc->head;
-				while (scan != NULL && req->interp==NULL) {
-					if (scan->enabled) {
-						if (scan->type == 'E') {
-							int suffixpos=strlen(req->path)-scan->len;
-							if (suffixpos>0 && strcmp(req->path+suffixpos,scan->magic)==0) {
-								req->interp=scan->interpreter;
-								req->flags=(strchr(scan->flags,'P') != NULL)?BINFMT_KEEP_ARG0:0;
-							}
-						} else if (scan->type == 'M') {
-							int i,j,diff;
-							/*for (i=scan->offset,j=0,diff=0;i<128 && j<scan->len && diff==0;i++,j++)
-								fprint2("%02x %02x %02x %2x\n",buf[i],scan->magic[j],scan->mask[j],
-										(buf[i] ^ scan->magic[j]) & scan->mask[j]);*/
-							for (i=scan->offset,j=0,diff=0;i<128 && j<scan->len && diff==0;i++,j++)
-								diff=(buf[i] ^ scan->magic[j]) & scan->mask[j];
-							if (diff==0) {
-								req->interp=scan->interpreter;
-								req->flags=(strchr(scan->flags,'P') != NULL)?BINFMT_KEEP_ARG0:0;
-							}
-						}
+	char *buf=req->buf;
+	if (fc->enabled) {
+		struct umregister *scan=fc->head;
+		while (scan != NULL && req->interp==NULL) {
+			if (scan->enabled) {
+				if (scan->type == 'E') {
+					int suffixpos=strlen(req->path)-scan->len;
+					if (suffixpos>0 && strcmp(req->path+suffixpos,scan->magic)==0) {
+						req->interp=scan->interpreter;
+						req->flags=(strchr(scan->flags,'P') != NULL)?BINFMT_KEEP_ARG0:0;
+					}
+				} else if (scan->type == 'M') {
+					int i,j,diff;
+					/*for (i=scan->offset,j=0,diff=0;i<128 && j<scan->len && diff==0;i++,j++)
+						printk("%02x %02x %02x %2x\n",buf[i],scan->magic[j],scan->mask[j],
+						(buf[i] ^ scan->magic[j]) & scan->mask[j]);*/
+					for (i=scan->offset,j=0,diff=0;i<128 && j<scan->len && diff==0;i++,j++)
+						diff=(buf[i] ^ scan->magic[j]) & scan->mask[j];
+					if (diff==0) {
+						req->interp=scan->interpreter;
+						req->flags=(strchr(scan->flags,'P') != NULL)?BINFMT_KEEP_ARG0:0;
 					}
-					scan=scan->next;
 				}
 			}
+			scan=scan->next;
 		}
+	}
+	//printk("searchbinfmt %s %s\n",req->path,req->interp);
 	if (req->interp!=NULL)
-		return get_epoch();
+		return 1;
 	else
 		return 0;
 }
 
-/*insert a new context in the mount table*/
-static struct umbinfmt *addmnttab(struct umbinfmt *new)
-{
-	register int i;
-	//pthread_mutex_lock( &mnttab_mutex );
-	for (i=0;i<mnttabmax && mnttab[i] != NULL;i++)
-		;
-	if (i>=mnttabmax) {
-		register int j;
-		register int mnttabnewmax=(i + MNTTABSTEP) & ~MNTTABSTEP_1;
-		mnttab=(struct umbinfmt **)realloc(mnttab,mnttabnewmax*sizeof(struct umbinfmt *));
-		assert(mnttab);
-		for (j=i;j<mnttabnewmax;j++)
-			mnttab[j]=NULL;
-		mnttabmax=mnttabnewmax;
-	}
-	mnttab[i]=new;
-	//pthread_mutex_unlock( &mnttab_mutex );
-	return mnttab[i];
-}
-
-/* execute a specific function (arg) for each mnttab element */
-static void forallmnttabdo(void (*fun)(struct umbinfmt *fc))
-{
-	register int i;
-	for (i=0;i<mnttabmax;i++)
-		if (mnttab[i] != NULL)
-			fun(mnttab[i]);
-}
-
-/*
- * delete the i-th element of the tab.
- * the table cannot be compacted as the index is used as id
- */
-static void delmnttab(struct umbinfmt *fc)
-{
-	register int i;
-	//pthread_mutex_lock( &mnttab_mutex );
-	for (i=0;i<mnttabmax && fc != mnttab[i];i++)
-		;
-	if (i<mnttabmax)
-		mnttab[i]=NULL;
-	else
-		fprint2("delmnt inexistent entry\n");
-	//pthread_mutex_unlock( &mnttab_mutex );
-}
-
-/* add an element to the filetab (open file table)
- * each file has a fileinfo record
- */
-static int addfiletab()
-{
-	register int i;
-	//pthread_mutex_lock( &mnttab_mutex );
-	for (i=0;i<filetabmax && filetab[i] != NULL;i++)
-		;
-	if (i>=filetabmax) {
-		register int j;
-		filetabmax=(i + MNTTABSTEP) & ~MNTTABSTEP_1;
-		filetab=(struct binfileinfo **)realloc(filetab,filetabmax*sizeof(struct binfileinfo *));
-		assert(filetab);
-		for (j=i;j<filetabmax;j++)
-			filetab[j]=NULL;
-	}
-	filetab[i]=(struct binfileinfo *)malloc(sizeof(struct binfileinfo));
-	assert(filetab[i]);
-	//pthread_mutex_unlock( &mnttab_mutex );
-	return i;
-}
-
-/* delete an entry from the open file table.
- * RD: there is a counter managed by open and close calls */
-static void delfiletab(int i)
-{
-	struct binfileinfo *norace=filetab[i];
-	filetab[i]=NULL;
-	free(norace);
-}
-
-
 #if 0
 #define MAXARGS 256
 
@@ -401,45 +260,67 @@ static struct umregister *delete_allreg(struct umregister *head)
 		return NULL;
 	}
 }
-		
+
+static int checkbinfmt(int type, void *arg, int arglen, struct ht_elem *ht)
+{
+	struct binfmt_req *req=arg;
+	void *private_data=ht_get_private_data(ht);
+	if (private_data)
+		return searchbinfmt(private_data,req);
+	else
+		return 0;
+}
+
 static long umbinfmt_mount(char *source, char *target, char *filesystemtype,
 		unsigned long mountflags, void *data)
 {
 	struct umbinfmt *new = (struct umbinfmt *) malloc(sizeof(struct umbinfmt));
 	assert(new);
 	new->path = strdup(target);
-	new->tst=tst_timestamp();
 	new->flags=(data && strcmp((char*)data,"debug")==0)?UMBINFMT_DEBUG:0;
 	new->inuse=0;
 	new->enabled=1;
 	new->head=NULL;
-	addmnttab(new);
+	if (strcmp(source,"none")==0 || strcmp(source,"/")==0)
+		new->binfmt_ht=ht_tab_add(CHECKBINFMT,NULL,0,&s,checkbinfmt,new);
+	else
+		new->binfmt_ht=ht_tab_add(CHECKBINFMT,source,strlen(source),&s,checkbinfmt,new);
+	ht_tab_pathadd(CHECKPATH,source,target,filesystemtype,mountflags,data,&s,0,NULL,new);
 	return 0;
 }
 
+static void umbinfmt_umount_internal(struct umbinfmt *fc, int flags)
+{
+	struct umbinfmt *fc_norace=fc;
+	char *target=fc->path;
+	if (fc_norace->flags & UMBINFMT_DEBUG) 
+		printk("UMOUNT => path:%s flag:%d\n",target, flags);
+	ht_tab_invalidate(fc->binfmt_ht);
+	ht_tab_invalidate(um_mod_get_hte());
+	delete_allreg(fc->head);
+	free(fc_norace->path);
+	free(fc_norace);
+}
+
 static long umbinfmt_umount2(char *target, int flags)
 {
-	struct umbinfmt *fc;
-	fc = searchbmfile(target);
+	struct umbinfmt *fc=um_mod_get_private_data();
 	if (fc == NULL) {
 		errno=EINVAL;
 		return -1;
 	} else if (fc->inuse > 0) {
 		errno=EBUSY;
 		return -1;
-	} else
-	{
-		struct umbinfmt *fc_norace=fc;
-		if (fc_norace->flags & UMBINFMT_DEBUG) 
-			fprint2("UMOUNT => path:%s flag:%d\n",target, flags);
-		delmnttab(fc);
-		delete_allreg(fc->head);
-		free(fc_norace->path);
-		free(fc_norace);
+	} else {
+		struct ht_elem *binfmt_ht=fc->binfmt_ht;
+		umbinfmt_umount_internal(fc,flags);
+		ht_tab_del(binfmt_ht);
+		ht_tab_del(um_mod_get_hte());
 		return 0;
 	}
 }
 
+#if 0
 #define TRUE 1
 #define FALSE 0
 
@@ -447,28 +328,7 @@ static int alwaysfalse()
 {
 	return FALSE;
 }
-
-static epoch_t umbinfmt_check(int type, void *arg)
-{
-	if (type == CHECKPATH) {
-		char *path=arg;
-		struct umbinfmt *fc=searchbmfile(path);
-		if ( fc != NULL) {
-			return TRUE; 
-		}
-		else
-			return FALSE;
-	} else if (type == CHECKFSTYPE) {
-		char *path=arg;
-		return ((strcmp(path,"umbinfmt") == 0) ||
-				(strcmp(path,"umbinfmt_misc") == 0));
-	} else if (type == CHECKBINFMT) {
-		struct binfmt_req *req=arg;
-		return searchbinfmt(req);
-	} else {
-		return FALSE;
-	}
-}
+#endif
 
 static char *unwrap(struct umbinfmt *fc,char *path)
 {
@@ -551,7 +411,8 @@ static char *hexstring(char *src,char *hex,int len)
 
 static char *createcontents(int fd,struct umbinfmt *fc,int *len)
 {
-	struct umregister *reg=filetab[fd]->reg;
+	struct binfileinfo *ft=getfiletab(fd);
+	struct umregister *reg=ft->reg;
 	assert (reg);
 	if (UBM_IS_ROOT(reg)) {
 		return create_dirent(fc,len);
@@ -587,8 +448,8 @@ static char *createcontents(int fd,struct umbinfmt *fc,int *len)
 
 static long umbinfmt_open(char *path, int flags, mode_t mode)
 {
-	struct umbinfmt *fc = searchbmfile(path);
-	int fi;
+	struct umbinfmt *fc = um_mod_get_private_data();
+	int fd;
 	int rv;
 	assert(fc!=NULL);
 	struct umregister *file=searchfile(path,fc);
@@ -599,8 +460,9 @@ static long umbinfmt_open(char *path, int flags, mode_t mode)
 			(UBM_IS_ROOT(file) && !(flags & O_DIRECTORY) && (flags & O_WRONLY)))
 		rv=-EINVAL;
 	else {
-		fi = addfiletab();
-		assert(fi>=0);
+		fd = addfiletab(sizeof(struct binfileinfo));
+		struct binfileinfo *ft=getfiletab(fd);
+		assert(fd>=0);
 
 #ifdef __UMBINFMT_DEBUG__
 		PRINTDEBUG(10,"FLAGOPEN path:%s \nFLAGS:0x%x MODE:%d\n",path,flags,mode);
@@ -640,15 +502,15 @@ static long umbinfmt_open(char *path, int flags, mode_t mode)
 		if(flags &  O_SYNC)
 			PRINTDEBUG(10, "SYNC\n");
 #endif
-		filetab[fi]->reg = file;
-		filetab[fi]->bfmount = fc;
-		filetab[fi]->flags = flags & ~(O_CREAT | O_EXCL | O_NOCTTY | O_TRUNC);
-		filetab[fi]->pos = 0;
+		ft->reg = file;
+		ft->bfmount = fc;
+		ft->flags = flags & ~(O_CREAT | O_EXCL | O_NOCTTY | O_TRUNC);
+		ft->pos = 0;
 		if (!(flags & O_WRONLY)) {
-			filetab[fi]->contents = createcontents(fi,fc,&(filetab[fi]->len));
+			ft->contents = createcontents(fd,fc,&(ft->len));
 		} else {
-			filetab[fi]->len = 0;
-			filetab[fi]->contents = NULL;
+			ft->len = 0;
+			ft->contents = NULL;
 		}
 		fc->inuse++;
 		rv=0;
@@ -657,13 +519,13 @@ static long umbinfmt_open(char *path, int flags, mode_t mode)
 	if (rv < 0)
 	{
 		if (fc->flags & UMBINFMT_DEBUG) 
-			fprint2("OPEN[%d] ERROR => path:%s flags:0x%x\n", fi, path, flags);	
+			printk("OPEN[%d] ERROR => path:%s flags:0x%x\n", fd, path, flags);	
 		errno = -rv;
 		return -1;
 	} else {
 		if (fc->flags & UMBINFMT_DEBUG) 
-			fprint2("OPEN[%d] => path:%s flags:0x%x\n", fi, path, flags);
-		return fi;
+			printk("OPEN[%d] => path:%s flags:0x%x\n", fd, path, flags);
+		return fd;
 	}
 }
 
@@ -671,27 +533,22 @@ static long umbinfmt_open(char *path, int flags, mode_t mode)
 
 static long umbinfmt_close(int fd)
 {
-	int rv;
+	struct binfileinfo *ft=getfiletab(fd);
 
-	if (filetab[fd]==NULL) {
-		errno=EBADF;
-		return -1;
+	if (ft->bfmount->flags & UMBINFMT_DEBUG) 
+		printk("CLOSE[%d]\n",fd);
+	if (ft->contents != NULL)
+		free(ft->contents);
+	ft->bfmount->inuse--;
+	if (UBM_IS_STATUS(ft->reg)) {
+		if (ft->bfmount->enabled == 0xff)
+			ft->bfmount->head=delete_allreg(ft->bfmount->head);
 	} else {
-		if (filetab[fd]->bfmount->flags & UMBINFMT_DEBUG) 
-			fprint2("CLOSE[%d]\n",fd);
-		if (filetab[fd]->contents != NULL)
-			free(filetab[fd]->contents);
-		filetab[fd]->bfmount->inuse--;
-		if (UBM_IS_STATUS(filetab[fd]->reg)) {
-			if (filetab[fd]->bfmount->enabled == 0xff)
-				filetab[fd]->bfmount->head=delete_allreg(filetab[fd]->bfmount->head);
-		} else {
-			if (filetab[fd]->reg->enabled == 0xff)
-				filetab[fd]->bfmount->head=delete_reg(filetab[fd]->bfmount->head,filetab[fd]->reg);
-		}
-		delfiletab(fd);
-		return 0;
+		if (ft->reg->enabled == 0xff)
+			ft->bfmount->head=delete_reg(ft->bfmount->head,ft->reg);
 	}
+	delfiletab(fd);
+	return 0;
 }
 
 static int count_dents64(void *buf, int count, int max) {
@@ -705,71 +562,43 @@ static int count_dents64(void *buf, int count, int max) {
 }
 
 static long umbinfmt_getdents64(int fd, void *buf, size_t count){
-	int rv;
-	if (filetab[fd]==NULL) {
-		errno=EBADF;
-		return -1;
-	} else if (! UBM_IS_ROOT(filetab[fd]->reg)) {
+	struct binfileinfo *ft=getfiletab(fd);
+	if (! UBM_IS_ROOT(ft->reg)) {
 		errno=ENOTDIR;
 		return -1;
 	}
 	else {
-		char *tail=(filetab[fd]->contents)+filetab[fd]->pos;
-		int rv=count_dents64(tail,count,filetab[fd]->len-filetab[fd]->pos);
-		memcpy(buf,(filetab[fd]->contents)+filetab[fd]->pos,rv);
+		char *tail=(ft->contents)+ft->pos;
+		int rv=count_dents64(tail,count,ft->len-ft->pos);
+		memcpy(buf,(ft->contents)+ft->pos,rv);
 		if (rv<0) {
 			errno= -rv;
 			return -1;
 		} else {
-			filetab[fd]->pos += rv;
+			ft->pos += rv;
 			return rv;
 		}
 	}
 }
 
-#if 0
-static void convert_dents6432(void *buf, int count) {
-	if (count > 0) {
-		struct umbinfmt_dirent *d32=(struct umbinfmt_dirent *)buf;
-		struct umbinfmt_dirent64 *d64=(struct umbinfmt_dirent64 *)buf;
-		void *next=(void *)(((char *)buf) + d64->d_reclen);
-		int nextcount=count - d64->d_reclen;
-		d32->d_ino=d64->d_ino;
-		d32->d_off=d64->d_off;
-		d32->d_reclen=d64->d_reclen;
-		//d32->d_type=d64->d_type;
-		memmove(d32->d_name,d64->d_name,strlen(d64->d_name)+1);
-		convert_dents6432(next,nextcount);
-	}
-}
-
-static long umbinfmt_getdents(int fd, void *buf, size_t count){
-	int rv=umbinfmt_getdents64(fd, buf, count);
-	convert_dents6432(buf,rv);
-	return rv;
-}
-#endif
-
 static long umbinfmt_read(int fd, void *buf, size_t count)
 {
 	int rv;
-	if (filetab[fd]==NULL) {
-		errno=EBADF;
-		return -1;
-	} else if (UBM_IS_ROOT(filetab[fd]->reg)) {
+	struct binfileinfo *ft=getfiletab(fd);
+	if (UBM_IS_ROOT(ft->reg)) {
 		errno=EISDIR;
 		return -1;
 	}
 	else {
 		rv = count;
-		if (rv > filetab[fd]->len - filetab[fd]->pos)
-			rv= filetab[fd]->len - filetab[fd]->pos;
-		strncpy(buf,(filetab[fd]->contents)+filetab[fd]->pos,rv);
+		if (rv > ft->len - ft->pos)
+			rv= ft->len - ft->pos;
+		strncpy(buf,(ft->contents)+ft->pos,rv);
 		if (rv<0) {
 			errno= -rv;
 			return -1;
 		} else {
-			filetab[fd]->pos += rv;
+			ft->pos += rv;
 			return rv;
 		}
 	}
@@ -813,7 +642,6 @@ static char *dechex(char *src,unsigned char *len)
 			src++;
 		}
 	}
-	buf[i];
 	if (*len == 0)
 		*len=i;
 	else
@@ -860,66 +688,42 @@ static void ubm_register(struct umbinfmt *fc,char *buf, size_t count)
 static long umbinfmt_write(int fd, void *buf, size_t count)
 {
 	int rv=count;
+	struct binfileinfo *ft=getfiletab(fd);
 	char *cbuf=buf;
 
-	if (filetab[fd]==NULL) {
-		errno = EBADF;
+	if (UBM_IS_REGISTER(ft->reg)) {
+		if (ft->pos == 0)
+			ubm_register(ft->bfmount,buf,count);
+	} 
+	else if (UBM_IS_STATUS(ft->reg)) {
+		if (count >= 1) { 
+			if (*cbuf=='1')
+				ft->bfmount->enabled = 1;
+			else if (*cbuf=='0')
+				ft->bfmount->enabled = 0;
+			if (count >= 2 && cbuf[0]=='-' && cbuf[1]=='1')
+				ft->bfmount->enabled = 0xff;
+		}
+	} else {
+		if (count >= 1) { 
+			if (*cbuf=='1')
+				ft->reg->enabled = 1;
+			else if (*cbuf=='0')
+				ft->reg->enabled = 0;
+			if (count >= 2 && cbuf[0]=='-' && cbuf[1]=='1')
+				ft->reg->enabled = 0xff;
+		}
+	}
+	if (rv<0) {
+		errno= -rv;
 		return -1;
 	} else {
-		if (UBM_IS_REGISTER(filetab[fd]->reg)) {
-			if (filetab[fd]->pos == 0)
-				ubm_register(filetab[fd]->bfmount,buf,count);
-		} 
-		else if (UBM_IS_STATUS(filetab[fd]->reg)) {
-			if (count >= 1) { 
-				if (*cbuf=='1')
-					filetab[fd]->bfmount->enabled = 1;
-				else if (*cbuf=='0')
-					filetab[fd]->bfmount->enabled = 0;
-				if (count >= 2 && cbuf[0]=='-' && cbuf[1]=='1')
-					filetab[fd]->bfmount->enabled = 0xff;
-			}
-		} else {
-			if (count >= 1) { 
-				if (*cbuf=='1')
-					filetab[fd]->reg->enabled = 1;
-				else if (*cbuf=='0')
-					filetab[fd]->reg->enabled = 0;
-				if (count >= 2 && cbuf[0]=='-' && cbuf[1]=='1')
-					filetab[fd]->reg->enabled = 0xff;
-			}
-		}
-		if (rv<0) {
-			errno= -rv;
-			return -1;
-		} else {
-			filetab[fd]->pos += rv;
-			return rv;
-		}
+		ft->pos += rv;
+		return rv;
 	}
 }
 
-/*
-static int stat2stat64(struct stat64 *s64, struct stat *s)
-{
-	s64->st_dev= s->st_dev;
-	s64->st_ino= s->st_ino;
-	s64->st_mode= s->st_mode;
-	s64->st_nlink= s->st_nlink;
-	s64->st_uid= s->st_uid;
-	s64->st_gid= s->st_gid;
-	s64->st_rdev= s->st_rdev;
-	s64->st_size= s->st_size;
-	s64->st_blksize= s->st_blksize;
-	s64->st_blocks= s->st_blocks;
-	s64->st_atim= s->st_atim;
-	s64->st_mtim= s->st_mtim;
-	s64->st_ctim= s->st_ctim;
-	return 0;
-}
-*/
-
-static int common_stat64(struct umbinfmt *fc,struct umregister *reg, struct stat64 *buf64)
+static inline int common_stat64(struct umbinfmt *fc,struct umregister *reg, struct stat64 *buf64)
 {
 	int rv;
 	if (reg == NULL) {
@@ -938,91 +742,32 @@ static int common_stat64(struct umbinfmt *fc,struct umregister *reg, struct stat
 		buf64->st_mode=S_IFREG | 0644;
 	rv=0;
 	if (fc->flags & UMBINFMT_DEBUG) 
-		fprint2("stat->GETATTR => status: %s\n",
+		printk("stat->GETATTR => status: %s\n",
 				rv ? "Error" : "Success");
 	return rv;
 }
 
-/*
-static int common_stat64(struct umbinfmt *fc,struct umregister *reg, struct stat64 *buf64)
+static long umbinfmt_lstat64(char *path, struct stat64 *buf64)
 {
-	int rv;
-	struct stat buf;
-	if ((rv=common_stat(fc,reg,&buf))>=0)
-		stat2stat64(buf64,&buf);
-	return rv;
-}
-*/
-
-#if 0
-/* 09 2008 f-syscall unification */
-static long umbinfmt_fstat64(int fd, struct stat64 *buf64)
-{
-	if (fd < 0 || filetab[fd] == NULL) {
-		errno=EBADF;
-		return -1;
-	} else {
-		return common_stat64(filetab[fd]->bfmount, filetab[fd]->reg,buf64);
-	}
-}
-#endif
-
-/*
-static long umbinfmt_fstat64(int fd, struct stat64 *buf64)
-{
-	if (filetab[fd]==NULL) {
-		errno=EBADF;
-		return -1;
-	} else {
-		int rv;
-		struct stat buf;
-		if ((rv=umbinfmt_fstat(fd,&buf))>=0)
-			stat2stat64(buf64,&buf);
-		return rv;
-	}
-}
-*/
-
-#if 0
-static long umbinfmt_stat(char *path, struct stat *buf)
-{
-	struct umbinfmt *umbinfmt=searchbmfile(path);
-	struct umregister *reg=searchfile(path,umbinfmt);
-	return common_stat(umbinfmt,reg,buf);
-}
-
-static long umbinfmt_lstat(char *path, struct stat *buf)
-{
-	return umbinfmt_stat(path,buf);
-}
-#endif
-
-static long umbinfmt_stat64(char *path, struct stat64 *buf64)
-{
-	struct umbinfmt *umbinfmt=searchbmfile(path);
+	struct umbinfmt *umbinfmt=um_mod_get_private_data();
 	struct umregister *reg=searchfile(path,umbinfmt);
 	return common_stat64(umbinfmt,reg,buf64);
 }
 
-static long umbinfmt_lstat64(char *path, struct stat64 *buf64)
-{
-	return umbinfmt_stat64(path,buf64);
-}
-
 static long umbinfmt_access(char *path, int mode)
 {
-	struct umbinfmt *fc=searchbmfile(path);
+	struct umbinfmt *fc=um_mod_get_private_data();
 	struct umregister *reg=searchfile(path,fc);
 	int rv;
 	assert(fc!=NULL);
 	if (fc->flags & UMBINFMT_DEBUG) 
-		fprint2("ACCESS => path:%s mode:%s%s%s%s\n", 
+		printk("ACCESS => path:%s mode:%s%s%s%s\n", 
 				path,
 				(mode & R_OK) ? "R_OK": "",
 				(mode & W_OK) ? "W_OK": "",
 				(mode & X_OK) ? "X_OK": "",
 				(mode & F_OK) ? "F_OK": "");
-	
+
 
 	if (UBM_IS_ROOT(reg))
 		rv= !(mode & W_OK); /* it is forbidden to create new file by hand*/
@@ -1049,25 +794,21 @@ static long umbinfmt_access(char *path, int mode)
 
 static loff_t umbinfmt_x_lseek(int fd, off_t offset, int whence)
 {
-	if (filetab[fd]==NULL) {
-		errno = EBADF; 
-		return -1;
-	} else {
-		switch (whence) {
-			case SEEK_SET:
-				filetab[fd]->pos=offset;
-				break;
-			case SEEK_CUR:
-				filetab[fd]->pos+=offset;
-				break;
-			case SEEK_END:
-				filetab[fd]->pos=filetab[fd]->len+offset;
-				break;
-		}
-		if (filetab[fd]->pos<0) filetab[fd]->pos=0;
-		if (filetab[fd]->pos > filetab[fd]->len) filetab[fd]->pos=filetab[fd]->len;
-		return filetab[fd]->pos;
+	struct binfileinfo *ft=getfiletab(fd);
+	switch (whence) {
+		case SEEK_SET:
+			ft->pos=offset;
+			break;
+		case SEEK_CUR:
+			ft->pos+=offset;
+			break;
+		case SEEK_END:
+			ft->pos=ft->len+offset;
+			break;
 	}
+	if (ft->pos<0) ft->pos=0;
+	if (ft->pos > ft->len) ft->pos=ft->len;
+	return ft->pos;
 }
 
 static long umbinfmt_lseek(int fd, int offset, int whence)
@@ -1095,19 +836,16 @@ static long umbinfmt__llseek(unsigned int fd, unsigned long offset_high,  unsign
 	}
 }
 
+#if 0
 static void contextclose(struct umbinfmt *fc)
 {
 	umbinfmt_umount2(fc->path,MNT_FORCE);
 }
+#endif
 
 static long umbinfmt_event_subscribe(void (* cb)(), void *arg, int fd, int how)
 {
-	if (filetab[fd]==NULL) {
-		errno=EBADF;
-		return -1;
-	}
-	else 
-		return 1;
+	return 1;
 }
 
 #if 0
@@ -1120,7 +858,7 @@ static epoch_t umbinfmt_check(int type, void *arg)
 			req->interp="/usr/local/bin/qemu-i386";
 			return get_epoch()-1;
 		} else
-					return 0;
+			return 0;
 	}
 	else 
 		return 0;
@@ -1132,58 +870,44 @@ static long umbinfmt_fcntl64()
 	return 0;
 }
 
+static void umbinfmt_destructor(int type,struct ht_elem *mp)
+{
+	switch (type) {
+		case CHECKPATH:
+			um_mod_set_hte(mp);
+			umbinfmt_umount_internal(um_mod_get_private_data(), MNT_FORCE);
+	}
+}
+
 	static void
 	__attribute__ ((constructor))
 init (void)
 {
-	printf("umbinfmt init\n");
+	printk(KERN_NOTICE "umbinfmt init\n");
 	s.name="umbinfmt";
-	s.code=UMBINFMT_SERVICE_CODE;
-	s.checkfun=umbinfmt_check;
+	s.description="virtual binfmt_misc";
+	s.destructor=umbinfmt_destructor;
 	s.syscall=(sysfun *)calloc(scmap_scmapsize,sizeof(sysfun));
 	s.socket=(sysfun *)calloc(scmap_sockmapsize,sizeof(sysfun));
 	SERVICESYSCALL(s, mount, umbinfmt_mount);
-#if 0
-#if ! defined(__x86_64__)
-	SERVICESYSCALL(s, umount, umbinfmt_umount2); /* umount must be mapped onto umount2 */
-#endif
-#endif
 	SERVICESYSCALL(s, umount2, umbinfmt_umount2);
 	SERVICESYSCALL(s, open, umbinfmt_open);
-#if 0
-	SERVICESYSCALL(s, creat, umbinfmt_open); /*creat is an open with (O_CREAT|O_WRONLY|O_TRUNC)*/
-#endif
 	SERVICESYSCALL(s, read, umbinfmt_read);
 	SERVICESYSCALL(s, write, umbinfmt_write);
 	SERVICESYSCALL(s, close, umbinfmt_close);
 #if !defined(__x86_64__)
-	SERVICESYSCALL(s, stat64, umbinfmt_stat64);
 	SERVICESYSCALL(s, lstat64, umbinfmt_lstat64);
-	//SERVICESYSCALL(s, fstat64, umbinfmt_fstat64);
 #else
-	SERVICESYSCALL(s, stat, umbinfmt_stat64);
 	SERVICESYSCALL(s, lstat, umbinfmt_lstat64);
-	//SERVICESYSCALL(s, fstat, umbinfmt_fstat64);
 #endif
 	SERVICESYSCALL(s, access, umbinfmt_access);
 	SERVICESYSCALL(s, lseek, umbinfmt_lseek);
 #if ! defined(__x86_64__)
 	SERVICESYSCALL(s, _llseek, umbinfmt__llseek);
-#endif
-#if 0
-	SERVICESYSCALL(s, getdents, umbinfmt_getdents);
 #endif
 	SERVICESYSCALL(s, getdents64, umbinfmt_getdents64);
-	SERVICESYSCALL(s, fcntl64, umbinfmt_fcntl64);
-	//SERVICESYSCALL(s, chown, umbinfmt_chown);
-	//SERVICESYSCALL(s, fchown, fchown);
-	//SERVICESYSCALL(s, chmod, umbinfmt_chmod);
-	//SERVICESYSCALL(s, fchmod, fchmod);
-	//SERVICESYSCALL(s, fsync, umbinfmt_fsync); 
-	//SERVICESYSCALL(s, _newselect, umbinfmt_select);
-	//SERVICESYSCALL(s, ioctl, umbinfmt_ioctl); 
+	SERVICESYSCALL(s, fcntl, umbinfmt_fcntl64);
 	s.event_subscribe=umbinfmt_event_subscribe;
-	add_service(&s);
 }
 
 	static void
@@ -1192,7 +916,6 @@ fini (void)
 {
 	free(s.syscall);
 	free(s.socket);
-	//	foralldevicetabdo(contextclose);
-	printf("umbinfmt fini\n");
+	printk(KERN_NOTICE "umbinfmt fini\n");
 }
 
