@@ -34,6 +34,7 @@
 static char mbrsignature[2]={0x55,0xAA};
 
 struct mbr_header {
+	char code[IDE_HEADER_OFFSET];
 	struct mbrpart {
 		unsigned char flags;
 		unsigned char chs_begin[3];
@@ -47,13 +48,13 @@ struct mbr_header {
 
 #define LE32_INT(X) (((X)[0])+(((X)[1])<<8)+(((X)[2])<<16)+(((X)[3])<<24))
 
-static void maxgeom(struct hd_geometry *geom, const char *chs)
+static void maxgeom(struct hd_geometry *geom, const unsigned char *chs)
 {
 	unsigned char s,h;
 	unsigned short c;
-	h=(unsigned char)chs[0];
-	s=((unsigned char)chs[1]) & 0x3f;
-	c=((unsigned char)chs[2]) + (((unsigned char)chs[1]) & 0xc0) << 2;
+	h=chs[0];
+	s=chs[1] & 0x3f;
+	c=chs[2] + ((chs[1] & 0xc0) << 2);
 	if ((h+1) > geom->heads)
 		geom->heads = h+1;
 	if (s > geom->sectors)
@@ -66,7 +67,7 @@ static void mbr_read(struct mbr *mbr)
 {
 	struct mbr_header mbr_header;
 	unsigned int ext_part_base=0;
-	pread64(mbr->fd, &mbr_header, sizeof(mbr_header), (off_t)IDE_HEADER_OFFSET);
+	pread64(mbr->fd, &mbr_header, sizeof(mbr_header), (off_t) 0);
 	if (memcmp(mbr_header.signature,mbrsignature,2) != 0) {
 		fprintf(stderr,"bad signature in MBR %x %x\n",
 				mbr_header.signature[0],mbr_header.signature[1]);
@@ -101,9 +102,9 @@ static void mbr_read(struct mbr *mbr)
 		/* Read the chain of logical partitions inside the extended partition */
 		while (ext_part_base > 0) {
 			off_t base=((off_t)(ext_part_base+offset)) << IDE_BLOCKSIZE_LOG;
-			pread64(mbr->fd, &mbr_header, sizeof(mbr_header), (base+IDE_HEADER_OFFSET));
+			pread64(mbr->fd, &mbr_header, sizeof(mbr_header), base);
 			if (memcmp(mbr_header.signature,mbrsignature,2) != 0) {
-				fprintf(stderr,"bad signature in block %d=%x %x\n",base,
+				fprintf(stderr,"bad signature in block %lld=%x %x\n", base,
 						mbr_header.signature[0],mbr_header.signature[1]);
 				ext_part_base=0;
 			} else {
