@@ -60,7 +60,7 @@ static int hdmbr_read(char type, dev_t device, char *buf, size_t len, loff_t pos
 				rv=pread64(mbr->fd,buf,len,pos);
 				return (rv<0)?-errno:rv;
 			} else
-				return -EINVAL;
+				return 0;
 		} else
 			return -ENODEV;
 	}
@@ -146,6 +146,7 @@ static int hdmbr_fini(char type, dev_t device, struct umdev *devhandle)
 {
 	struct mbr *mbr=umdev_getprivatedata(devhandle);
 	mbr_close(mbr);
+	return 0;
 }
 
 static int hdmbr_ioctl(char type, dev_t device, int req, void * arg, struct dev_info *di)
@@ -177,7 +178,7 @@ static int hdmbr_ioctl(char type, dev_t device, int req, void * arg, struct dev_
 		case BLKGETSIZE64: {
 												 int partno=minor(device)-minor(umdev_getbasedev(di->devhandle));
 												 if (partno==0)
-													 *(long long *)arg = (mbr->size >> IDE_BLOCKSIZE_LOG);
+													 *(long long *)arg = mbr->size;
 												 else { /* access a partition */
 													 struct partition *partition=mbr->part[partno-1];
 													 if (partition) 
@@ -197,7 +198,7 @@ static int hdmbr_ioctl(char type, dev_t device, int req, void * arg, struct dev_
 	return 0;
 }
 
-static int hdmbr_ioctl_params(char type, dev_t device, int req, struct umdev *devhandle)
+static int hdmbr_ioctl_params(char type, dev_t device, int req, struct dev_info *di)
 {
 	switch (req) {
 		/*case BLKROSET: return (sizeof(int) | IOCTL_R);
