@@ -18,7 +18,7 @@
  *   with this program; if not, write to the Free Software Foundation, Inc.,
  *   51 Franklin St, Fifth Floor, Boston, MA  02110-1301 USA.
  *
- *   $Id: umfusestd.h 610 2008-08-29 22:44:47Z rd235 $
+ *   $Id: umfusestd.h 787 2009-09-11 14:46:23Z rd235 $
  *
  */
 #ifndef __UM_FUSESTD_H
@@ -28,8 +28,10 @@
 #define FUSE_DEBUG       (1 << 29)
 /** Enable human mode */
 #define FUSE_HUMAN       (1 << 28)
-/** Enable human mode */
+/** Enable merge mode */
 #define FUSE_MERGE       (1 << 27)
+/** Enable hard remove */
+#define FUSE_HARDREMOVE  (1 << 26)
 
 extern struct fuse_operations defaultservice;
 
