@@ -18,7 +18,7 @@
  *   with this program; if not, write to the Free Software Foundation, Inc.,
  *   51 Franklin St, Fifth Floor, Boston, MA  02110-1301 USA.
  *
- *   $Id: capture_km.h 668 2009-05-30 15:08:31Z rd235 $
+ *   $Id: capture_km.h 866 2010-08-11 07:17:33Z rd235 $
  *
  */
 #ifndef CAPTURE_KM_H_
@@ -49,6 +49,20 @@ int pcbtablesize(void);
 /* This is the handler of sigchld from user processes */
 void tracehand(void *useless);
 
+/* pcb search (linear scan) */
+struct pcb *pid2pcb(int pid);
+
+/* chroot or virtual cwd for one process */
+void capture_km_kmpid_chroot(pid_t kmpid,int onoff);
+
+/* zerovirt optimiation */
+void capture_km_global_get_path_syscalls(void);
+
+void capture_km_global_skip_path_syscalls(void);
+
+int ghosthash_add(const char *path,int len);
+int ghosthash_del(const char *path,int len);
+
 int capture_attach(struct pcb *pc,pid_t pid);
 
 #endif
