@@ -26,7 +26,7 @@
  *   with this program; if not, write to the Free Software Foundation, Inc.,
  *   51 Franklin St, Fifth Floor, Boston, MA  02110-1301 USA.
  *
- *   $Id: capture_um.c 668 2009-05-30 15:08:31Z rd235 $
+ *   $Id: capture_um.c 1011 2011-10-24 20:10:26Z rd235 $
  *
  */   
 #include <assert.h>
@@ -64,27 +64,34 @@ pthread_key_t pcb_key=0; /* key to grab the current thread pcb */
 sfun native_syscall=syscall;
 
 /* debugging output, (bypass pure_libc when loaded) */
-int fprint2(const char *fmt, ...) {
+int vprintk(const char *fmt, va_list ap) {
 	char *s;
-	int rv;
-	va_list ap;
-	va_start(ap,fmt);
-	rv=vasprintf(&s, fmt, ap);
-	va_end(ap);
-	if (rv>0)
-		rv=r_write(2,s,strlen(s));
-	free(s);
+	int rv=0;
+	int level=PRINTK_STANDARD_LEVEL;
+	if (fmt[0] == '<' && fmt[1] != 0 && fmt[2] == '>') {
+		/*level*/
+		switch (fmt[1]) {
+			case '0' ... '7':
+				level=fmt[1] - '0';
+				fmt+=3;
+				break;
+		}
+	}
+	if (level <= printk_current_level) {
+		rv=vasprintf(&s, fmt, ap);
+		if (rv>0)
+			rv=r_write(2,s,strlen(s));
+		free(s);
+	}
 	return rv;
 }
 
-int vfprint2(const char *fmt, va_list ap) {
-	char *s;
+int printk(const char *fmt, ...) {
 	int rv;
-	rv=vasprintf(&s, fmt, ap);
+	va_list ap;
+	va_start(ap,fmt);
+	rv=vprintk(fmt,ap);
 	va_end(ap);
-	if (rv>0)
-		rv=r_write(2,s,strlen(s));
-	free(s);
 	return rv;
 }
 
@@ -123,11 +130,6 @@ static char socketcallnargs[] = {
 };
 #endif
 
-/* When a SIGCHLD is received, the main select will be notified through this
- * pipe; the counter is used to notify it no more than one time. */
-/* if the kernel supports ppoll tracerpipe is not used */
-int tracerpipe[2];
-
 /* umview have to exit with the exit code of the first child: here we remember
  * what the first child was, and save its exit value */
 pid_t first_child_pid;
@@ -224,7 +226,7 @@ void forallpcbdo(voidfun f,void *arg)
 }
 
 /* pid 2 pcb conversion (by linear search) */
-static struct pcb *pid2pcb(int pid)
+struct pcb *pid2pcb(int pid)
 {
 	register int i;
 	for (i = 0; i < pcbtabsize; i++) {
@@ -249,6 +251,8 @@ static void droppcb(struct pcb *pc)
 	 * the termination of all modules */
 	/* otherwise the "nesting" mechanism misunderstands
 	 * the pcb by a npcb */
+	/* XXX rd235 20090805: it seems not a problem any more
+		 in the new version. deleted for dup delproc notication for proc #1 */
 #ifdef _PROC_MEM_TEST
 	if (pc->memfd >= 0)
 		close(pc->memfd);
@@ -256,7 +260,9 @@ static void droppcb(struct pcb *pc)
 	nprocs--;
 	forallpcbdo(_cut_pp,pc);
 	pcb_destructor(pc,0/*flags*/,0);
+#if 0
 	if (nprocs > 0)
+#endif
 		pc->flags = 0; /*NOT PCB_INUSE */;
 }
 
@@ -280,12 +286,14 @@ static void allocatepcbtab()
 static int handle_new_proc(int pid, struct pcb *pp)
 {
 	struct pcb *oldpc,*pc;
+#ifdef LIBC_VFORK_DIRTY_TRICKS
 	long saved_regs[VIEWOS_FRAME_SIZE];
+#endif /*LIBC_VFORK_DIRTY_TRICKS*/
 
-	//fprint2("handle_new_proc %d %p\n",pid,pp);
+	//printk("handle_new_proc %d %p\n",pid,pp);
 	if ((oldpc=pc=pid2pcb(pid)) == NULL && (pc = newpcb(pid))== NULL) {
-		fprint2("[pcb table full]\n");
-		if(ptrace(PTRACE_KILL, pid, 0, 0) < 0){
+		printk("[pcb table full]\n");
+		if(r_ptrace(PTRACE_KILL, pid, 0, 0) < 0){
 			GPERROR(0, "KILL");
 			exit(1);
 		}
@@ -300,13 +308,13 @@ static int handle_new_proc(int pid, struct pcb *pp)
 			getregs(pc);
 			putargn(0,pp->sysargs[0],pc);
 			putargn(1,pp->sysargs[1],pc);
-			////fprint2("starting1 %x %x was %x %x\n",pp->sysargs[0],pp->sysargs[1],getargn(0,pc),getargn(1,pc));
-			if(setregs(pc,PTRACE_SYSCALL,0,SIGSTOP) < 0){
+			////printk("starting1 pc->pid %d  %x %x was %x %x\n",pc->pid, pp->sysargs[0],pp->sysargs[1],getargn(0,pc),getargn(1,pc));
+			if(setregs(pc,PTRACE_SYSCALL,0,0) < 0){
 				GPERROR(0, "continuing");
 				exit(1);
 			}
 #else
-			if(ptrace(PTRACE_SYSCALL, pid, 0, SIGSTOP) < 0){
+			if(r_ptrace(PTRACE_SYSCALL, pid, 0, 0) < 0){
 				GPERROR(0, "continuing");
 				exit(1);
 			}
@@ -344,7 +352,7 @@ int fakesigstopcont(struct pcb *pc)
 	{
 		GDEBUG(1, "FAKECONT %d",kpid);
 		kpc->flags &= ~PCB_FAKESTOP;
-		if(ptrace(PTRACE_SYSCALL, kpid, 0, 0) < 0){
+		if(r_ptrace(PTRACE_SYSCALL, kpid, 0, 0) < 0){
 			GPERROR(0, "continuing");
 			exit(1);
 		}
@@ -360,7 +368,7 @@ void offspring_enter(struct pcb *pc)
 	//printf("offspring_enter:%d\n",pc->pid);
 	pc->sysargs[0]=getargn(0,pc);
 	pc->sysargs[1]=getargn(1,pc);
-	//fprint2("offspring_enter %x %x\n",pc->sysargs[0],pc->sysargs[1]);
+	//printk("offspring_enter %x %x\n",pc->sysargs[0],pc->sysargs[1]);
 	if (pc->sysscno == __NR_fork || pc->sysscno == __NR_vfork) {
 		putscno(__NR_clone,pc);
 		putargn(0,CLONE_PTRACE|SIGCHLD, pc);
@@ -400,14 +408,14 @@ void tracehand()
 			if(WIFSTOPPED(status) && (WSTOPSIG(status) == SIGSTOP)) {
 				/* create the descriptor, block the process
 				 * until the parent complete the pcb */
-				////fprint2("RACE CONDITION %d\n",pid);
+				////printk("RACE CONDITION %d\n",pid);
 				handle_new_proc(pid,NULL);
 				continue;
 			}
 			/* error case */
-			fprint2("signal from unknown pid %d: killed\n",pid);
+			printk("signal from unknown pid %d: killed\n",pid);
 			GDEBUG(0, "signal from unknown pid %d: killed",pid);
-			if(ptrace(PTRACE_KILL, pid, 0, 0) < 0){
+			if(r_ptrace(PTRACE_KILL, pid, 0, 0) < 0){
 				GPERROR(0, "KILL");
 				exit(1);
 			}
@@ -482,7 +490,7 @@ void tracehand()
 				}
 #if __NR_socketcall != __NR_doesnotexist
 				if (scno==__NR_socketcall) {
-					//fprint2("socketcall %d %x\n",pc->sysargs[0],pc->sysargs[1]);
+					//printk("socketcall %d %x\n",pc->sysargs[0],pc->sysargs[1]);
 					pc->sysscno=pc->sysargs[0];
 					pc->sockaddr=pc->sysargs[1];
 					umoven(pc,pc->sockaddr,
@@ -540,16 +548,22 @@ void tracehand()
 			} else { /* POST syscall management (OUT phase) */
 				divfun fun;
 				GDEBUG(3, "<-- pid %d syscall %d (%s) @ %p", pid, scno, SYSCALLNAME(scno), getpc(pc));
-				//fprint2("OUT\n");
+				//printk("OUT\n");
 				if (isreproducing) {
 					long newpid;
 					newpid=getrv(pc);
-					handle_new_proc(newpid,pc);
+					if (newpid >= 0) {
+						handle_new_proc(newpid,pc);
+						offspring_exit(pc);
+						putrv(newpid,pc);
+					} else {
+						////printf("ERESTARTNOINTR scno %d %ld %ld\n",scno, newpid,pc->saved_regs[MY_RAX]);
+						offspring_exit(pc);
+					}
+
 					GDEBUG(3, "FORK! %d->%d",pid,newpid);
 
 					/* restore original arguments */
-					offspring_exit(pc);
-					putrv(newpid,pc);
 				}
 				/* It is just for the sake of correctness, this test could be
 				 * safely eliminated  to increase the performance*/
@@ -586,7 +600,7 @@ void tracehand()
 			if ((pc->behavior & SC_SUSPENDED) == 0) {
 				if ((pc->behavior & SC_SAVEREGS) || isreproducing) {
 					if (PT_VM_OK) {
-						/*fprint2("SC %s %d\n",SYSCALLNAME(scno),pc->behavior);*/
+						/*printk("SC %s %d\n",SYSCALLNAME(scno),pc->behavior);*/
 						if(setregs(pc,PTRACE_SYSVM, (isreproducing ? 0 : (pc->behavior & SC_VM_MASK)),pc->signum) == -1)
 							GPERROR(0, "setregs");
 						if(!isreproducing && (pc->behavior & PTRACE_VM_SKIPEXIT))
@@ -596,14 +610,14 @@ void tracehand()
 							GPERROR(0, "setregs");
 				} else /* register not modified */
 				{
-					//fprint2 ("RESTART\n");
+					//printk ("RESTART\n");
 					if (PT_VM_OK) {
-						if (ptrace(PTRACE_SYSVM,pc->pid,pc->behavior & SC_VM_MASK,pc->signum) < 0)
+						if (r_ptrace(PTRACE_SYSVM,pc->pid,pc->behavior & SC_VM_MASK,pc->signum) < 0)
 							GPERROR(0, "restart");
 						if(pc->behavior & PTRACE_VM_SKIPEXIT)
 							pc->sysscno=NOSC; 
 					}else {
-						if (ptrace(PTRACE_SYSCALL,pc->pid,0,pc->signum) < 0)
+						if (r_ptrace(PTRACE_SYSCALL,pc->pid,0,pc->signum) < 0)
 							GPERROR(0, "restart");
 					}
 				}
@@ -633,7 +647,7 @@ void tracehand()
 			}
 			/*if (!sigishandled(pc,  WSTOPSIG(status))) {
 			// also progenie, but for now 
-			//ptrace(PTRACE_KILL,pid,0,0);
+			//r_ptrace(PTRACE_KILL,pid,0,0);
 			//printf("KILLED %d %d\n", pid,pc->pid);
 			}*/
 #ifdef FAKESIGSTOP
@@ -648,16 +662,16 @@ void tracehand()
 					pc->flags &= ~PCB_STARTING;
 #ifdef LIBC_VFORK_DIRTY_TRICKS
 					getregs(pc);
-					////fprint2("starting2 %x %x was %x %x\n",pc->sysargs[0],pc->sysargs[1],getargn(0,pc),getargn(1,pc));
+					////printk("starting2 %x %x was %x %x\n",pc->sysargs[0],pc->sysargs[1],getargn(0,pc),getargn(1,pc));
 					putargn(0,pc->sysargs[0],pc);
 					putargn(1,pc->sysargs[1],pc);
-					setregs(pc,PTRACE_SYSCALL,0,SIGSTOP);
+					setregs(pc,PTRACE_SYSCALL,0,0);
 #else
-					ptrace(PTRACE_SYSCALL, pid, 0, SIGSTOP);
+					r_ptrace(PTRACE_SYSCALL, pid, 0, 0);
 #endif
 				} else
 					/* forward signals to the process */
-					if(ptrace(PTRACE_SYSCALL, pid, 0, WSTOPSIG(status)) < 0){
+					if(r_ptrace(PTRACE_SYSCALL, pid, 0, WSTOPSIG(status)) < 0){
 						GPERROR(0, "continuing");
 						exit(1);
 					}
@@ -681,7 +695,7 @@ void tracehand()
 void sc_resume(struct pcb *pc)
 {
 	/* int pid=pc->pid; */
-	//fprint2("RESUME %d\n",pc->pid);
+	//printk("RESUME %d\n",pc->pid);
 	int scno=pc->sysscno;
 	int inout=pc->behavior-SC_SUSPENDED;
 	int	isreproducing=(scno == __NR_fork ||
@@ -740,7 +754,7 @@ void sc_resume(struct pcb *pc)
 			pc->behavior = SC_SUSPOUT;
 		signum=pc->signum;
 	}
-	//fprint2("RESTARTED %d %d\n",pc->pid,pc->behavior);
+	//printk("RESTARTED %d %d\n",pc->pid,pc->behavior);
 	/* restore registers and restart ONLY IF the call is not already blocking */
 	if ((pc->behavior & SC_SUSPENDED) == 0) {
 		if (PT_VM_OK) {
@@ -756,46 +770,14 @@ void sc_resume(struct pcb *pc)
 	}
 }
 
-/*
- * Set up the pipe used by the SIGCHLD signal handler to wake the main
- * select() and tell it to start tracehand().
- * (ONLY used when ppoll is not a valid syscall)
- */
-int wake_tracer_init()
-{
-	r_pipe(tracerpipe);
-	r_fcntl(tracerpipe[0],F_SETFL,O_NONBLOCK);
-	return(tracerpipe[0]);
-}
-
-/*
- * Write data to the tracerpipe: we received a SIGCHLD and the main
- * select cycle must run tracehand()
- * (ONLY used when ppoll is not a valid syscall)
- */
-void wake_tracer(int s)
-{
-	char x = 0;
-	r_write(tracerpipe[1], &x, 1);
-}
-
-void do_wake_tracer()
-{
-	{
-		char buf[256];
-		r_read(tracerpipe[0], buf, 256);
-	}
-	tracehand();
-}
-
-/* fake function used when ppoll is a valid system call */
 void wake_null(int s)
 {
 }
 
-static void setsigaction(int has_ppoll)
+static void setsigaction(void)
 {
 	struct sigaction sa;
+	sigset_t blockchild; 
 
 	sa.sa_handler = SIG_IGN;
 	sigemptyset(&sa.sa_mask);
@@ -819,14 +801,10 @@ static void setsigaction(int has_ppoll)
 	 * With ppoll there is no need for pipe: in this latter
 	 * case SIGCHLD gets blocked. SIGCHLD will unblock ppoll
 	 */
-	if (has_ppoll==1) {
-		sigset_t blockchild; 
-		sigemptyset(&blockchild);
-		sigaddset(&blockchild,SIGCHLD);
-		r_sigprocmask(SIG_BLOCK,&blockchild,NULL);
-		sa.sa_handler = wake_null;
-	} else 
-		sa.sa_handler = wake_tracer;
+	sigemptyset(&blockchild);
+	sigaddset(&blockchild,SIGCHLD);
+	r_sigprocmask(SIG_BLOCK,&blockchild,NULL);
+	sa.sa_handler = wake_null;
 	r_sigaction(SIGCHLD, &sa, NULL);
 }
 
@@ -880,12 +858,12 @@ static int r_execvp(const char *file, char *const argv[]){
 int capture_attach(struct pcb *pc,pid_t pid)
 {
 	handle_new_proc(pid,pc);
-	if (ptrace(PTRACE_ATTACH,pid,0,0) < 0)
+	if (r_ptrace(PTRACE_ATTACH,pid,0,0) < 0)
 		return -errno;
 	else {
 		int status;
 		if(r_waitpid(pid, &status, WUNTRACED) < 0 ||
-				ptrace(PTRACE_SYSCALL, pid, 0, 0) < 0)
+				r_ptrace(PTRACE_SYSCALL, pid, 0, 0) < 0)
 			GPERROR(0, "restarting attached");
 		return 0;
 	}
@@ -908,7 +886,7 @@ void capture_execrc(const char *path,const char *argv1)
 }
 
 /* main capture startup */
-int capture_main(char **argv,int has_ppoll,char *rc)
+int capture_main(char **argv, char *rc)
 {
 	int status;
 #if __NR_socketcall != __NR_doesnotexist
@@ -922,12 +900,7 @@ int capture_main(char **argv,int has_ppoll,char *rc)
 			exit(1);
 			break;
 		case 0:
-			/* FIRST PROCESS STARTUP */
-			/* no pipes are needed when the system supports ppoll */
-			if (!has_ppoll) {
-				close(tracerpipe[0]);
-				close(tracerpipe[1]);
-			} else {
+			{
 				sigset_t unblockall;
 				sigemptyset(&unblockall);
 				r_sigprocmask(SIG_SETMASK,&unblockall,NULL);
@@ -936,7 +909,7 @@ int capture_main(char **argv,int has_ppoll,char *rc)
 			/* try to set process priority back to standard prio (effective only when 
 			 * umview runs in setuid mode), useless call elsewhere */
 			r_setpriority(PRIO_PROCESS,0,0);
-			if(ptrace(PTRACE_TRACEME, 0, 0, 0) < 0){
+			if(r_ptrace(PTRACE_TRACEME, 0, 0, 0) < 0){
 				GPERROR(0, "ptrace");
 				exit(1);
 			}
@@ -965,9 +938,9 @@ int capture_main(char **argv,int has_ppoll,char *rc)
 				exit(1);
 			}
 			/* set up the signal management */
-			setsigaction(has_ppoll);
+			setsigaction();
 			/* okay, the first process can start (traced) */
-			if(ptrace(PTRACE_SYSCALL, first_child_pid, 0, 0) < 0){
+			if(r_ptrace(PTRACE_SYSCALL, first_child_pid, 0, 0) < 0){
 				GPERROR(0, "continuing");
 				exit(1);
 			}
