@@ -20,13 +20,14 @@
  *   with this program; if not, write to the Free Software Foundation, Inc.,
  *   51 Franklin St, Fifth Floor, Boston, MA  02110-1301 USA.
  *
- *   $Id: defs.h 669 2009-05-30 16:24:08Z rd235 $
+ *   $Id: defs.h 974 2011-08-08 08:52:20Z rd235 $
  *
  */   
 #ifndef _DEFS_H
 #define _DEFS_H
 #include <sys/syscall.h>
 #include <unistd.h>
+#include <stdlib.h>
 #include <endian.h>
 #include <stdarg.h>
 #include <gdebug.h>
@@ -36,7 +37,9 @@
 #include "nrsyscalls.h"
 #include <sys/ptrace.h>
 #include <sys/utsname.h>
+#include <sys/mount.h>
 #include <asm/ptrace.h>
+#include <sys/capability.h>
 
 #define umNULL ((long) NULL)
 #undef SNDRCVMSGUNIFY
@@ -61,6 +64,11 @@
 #endif
 // #define FAKESIGSTOP
 
+#ifndef MS_KERNMOUNT
+#define MS_KERNMOUNT    (1<<22) /* this is a kern_mount call */
+#endif
+#define MS_GHOST MS_KERNMOUNT /* ghost mount */
+
 #ifdef _VIEWOS_KM
 #	define sysargs	event.args
 #	define sockargs event.args
@@ -72,6 +80,7 @@
 #	define putrv(RV,PC) ( (PC)->outevent.retval = (RV) )
 #	define geterrno(PC) ( (PC)->outevent.erno )
 #	define puterrno(ERR,PC) ( (PC)->outevent.erno = (ERR) )
+#	define puterrno0(PC) puterrno(0,(PC))
 #	define getsp(PC) ((PC)->event.sp)
 #	define getpc(PC) ((PC)->event.pc)
 #	define putsp(RV,PC) ( (PC)->event.sp=(RV) )
@@ -103,11 +112,14 @@ extern sfun native_syscall;
 #define r_lstat64(p,b) (native_syscall(NR64_lstat,(p),(b)))
 #define r_readlink(p,b,sz) (native_syscall(__NR_readlink,(p),(b),(sz)))
 #define r_fcntl(f,c,a) (native_syscall(__NR_fcntl,(f),(c),(a)))
+#ifdef __NR_fcntl64
+#define r_fcntl64(f,c,a) (native_syscall(__NR_fcntl64,(f),(c),(a)))
+#endif
 #define r_umask(m) (native_syscall(__NR_umask,(m)))
 #define r_pipe(v) (native_syscall(__NR_pipe,(v)))
 #define r_access(p,m) (native_syscall(__NR_access,(p),(m)))
 #define r_setpriority(w,p,o) (native_syscall(__NR_setpriority,(w),(p),(o)))
-#define r_setuid(u) (native_syscall(__NR_setuid,(u)))
+#define r_setuid(u) (native_syscall(__NR_setresuid,(u),(u),(u)))
 #define r_getuid() (native_syscall(__NR_getuid))
 #define r_getpid() (native_syscall(__NR_getpid))
 /* be careful getcwd syscall does not allocate the string for path=NULL */
@@ -117,6 +129,8 @@ extern sfun native_syscall;
 #define r_kill(p,s) (native_syscall(__NR_kill,(p),(s)))
 #define r_execve(p,a,e) (native_syscall(__NR_execve,(p),(a),(e)))
 #define r_lseek(f,o,w) (native_syscall(__NR_close,(f),(o),(w)))
+#define r_getresuid(r,e,s) (native_syscall(__NR_getresuid,(r),(e),(s)))
+#define r_getresgid(r,e,s) (native_syscall(__NR_getresgid,(r),(e),(s)))
 #if 0
 #define r_sigsuspend(m) (native_syscall(__NR_sigsuspend,(m)))
 #define r_sigaction(s,a,o) (native_syscall(__NR_sigaction,(s),(a),(o)))
@@ -143,13 +157,38 @@ extern sfun native_syscall;
 #define r_pread64(f,b,c,o1,o2) (native_syscall(__NR_pread64,(f),(b),(c),__LONG_LONG_PAIR((o1),(o2))))
 #define r_pwrite64(f,b,c,o1,o2) (native_syscall(__NR_pwrite64,(f),(b),(c),__LONG_LONG_PAIR((o1),(o2))))
 #endif
+#ifdef __NR_getgroups32
+#define r_getgroups(s,g) (native_syscall(__NR_getgroups32,(s),(g)))
+#define r_setgroups(s,g) (native_syscall(__NR_setgroups32,(s),(g)))
+#else
+#define r_getgroups(s,g) (native_syscall(__NR_getgroups,(s),(g)))
+#define r_setgroups(s,g) (native_syscall(__NR_setgroups,(s),(g)))
+#endif
+#define r_ptrace(r,p,a,d) (native_syscall(__NR_ptrace,(r),(p),(a),(d)))
+#define r_tkill(t,s) (native_syscall(__NR_tkill,(t),(s)))
+#define r_tgkill(t,g,s) (native_syscall(__NR_tgkill,(t),(g),(s)))
 
 /* debugging functions */
-extern int fprint2(const char *fmt, ...);
-extern int vfprint2(const char *fmt, va_list ap);
+#define KERN_EMERG      "<0>"   /* system is unusable                   */
+#define KERN_ALERT      "<1>"   /* action must be taken immediately     */
+#define KERN_CRIT       "<2>"   /* critical conditions                  */
+#define KERN_ERR        "<3>"   /* error conditions                     */
+#define KERN_WARNING    "<4>"   /* warning conditions                   */
+#define KERN_NOTICE     "<5>"   /* normal but significant condition     */
+#define KERN_INFO       "<6>"   /* informational                        */
+#define KERN_DEBUG      "<7>"   /* debug-level messages                 */
+
+#define PRINTK_STANDARD_LEVEL 4
+#define PRINTK_STARTUP_LEVEL 5
+#define PRINTK_QUIET_LEVEL 4
+extern int printk(const char *fmt, ...);
+extern int vprintk(const char *fmt, va_list ap);
 
 /* verbosity/quietness */
 extern unsigned int quiet;
+extern unsigned int printk_current_level;
+/* omnipotent/human */
+extern unsigned int secure;
 
 #ifdef _VIEWOS_UM
 	/* flags on the underlying kernel support */
@@ -164,6 +203,37 @@ extern unsigned int quiet;
 #define WORDLEN sizeof(int *)
 #define WORDALIGN(X) (((X) + WORDLEN) & ~(WORDLEN-1))
 
+#define MAX_SOCKET_NAME 1024
+#define MAX_SOCKOPT_LEN 4096
+#define MAX_SOCK_CONTROLLEN (1<<16) /*64K*/
+#define _LARGE_ALLOCA_PROTECTION
+#ifdef _LARGE_ALLOCA_PROTECTION
+#define lalloca(L) ({ void *m; if (__builtin_expect(((L)>>16),0)) \
+		{ if ((m=malloc(L))==NULL) \
+		{ pc->retval= -1; pc->erno=ENOMEM; return SC_FAKE; } \
+	 	} \
+		else m=alloca(L); m; })
+#define lnalloca(L) ({ void *m; if (__builtin_expect(((L)>>16),0)) \
+		{ if ((m=malloc(L))==NULL) \
+		{ errno=ENOMEM; return -1; } \
+		} \
+		else m=alloca(L); m; })
+#define lfree(B,L) ({if (__builtin_expect(((L)>>16),0)) free(B); })
+#define lfree(B,L) ({if (__builtin_expect(((L)>>16),0)) free(B); })
+#else
+#define lalloca(L) alloca(L)
+#define lnalloca(L) alloca(L)
+#define lfree(B,L)
+#endif
+
+/* there is a memory alignment problem in these architectures */
+/* long long syscall args are 2 regs aligned */
+#if (defined(__powerpc__) && !defined(__powerpc64__)) || (defined (MIPS) && !defined(__mips64))
+#define PALIGN 1
+#else
+#define PALIGN 0
+#endif
+
 #if 0
 #ifdef _MALLOC_DEBUG
 #define free(X) ({ printf("MDBG-FREE %x %s %d\n",(X),__FILE__,__LINE__); \
@@ -193,6 +263,9 @@ typedef	int (*divfun)(int sc_number,int inout,struct pcb *ppcb);
 
 //#####################################
 // SYSCALL STRANGE STUFF
+// define __NR_doesnotexist all non-existent syscall
+// in this way the sequence in scmap is preserved through
+// different architectures
 #define __NR_doesnotexist -1
 #if defined(__x86_64__)
 #define NR64_stat	__NR_stat
@@ -219,12 +292,49 @@ typedef	int (*divfun)(int sc_number,int inout,struct pcb *ppcb);
 
 /* UNAME HISTORY */
 #ifndef __NR_oldolduname
-#define __NR_oldolduname __NR_uname
+#define __NR_oldolduname __NR_doesnotexist
 #endif
 #ifndef __NR_olduname
-#define __NR_olduname __NR_uname
+#define __NR_olduname __NR_doesnotexist
 #endif
 
+#ifndef __NR_dup3
+#define __NR_dup3 __NR_doesnotexist
+#endif
+
+#ifndef __NR_openat
+#define __NR_openat __NR_doesnotexist
+#define __NR_mkdirat __NR_doesnotexist
+#define __NR_mknodat __NR_doesnotexist
+#define __NR_fchownat __NR_doesnotexist
+#define __NR_futimesat  __NR_doesnotexist
+#endif
+
+#ifndef __NR_utimensat
+#define __NR_utimensat __NR_doesnotexist
+#endif
+
+#ifndef __NR_getxattr
+#define __NR_getxattr __NR_doesnotexist
+#define __NR_lgetxattr __NR_doesnotexist
+#define __NR_fgetxattr __NR_doesnotexist
+#define __NR_setxattr __NR_doesnotexist
+#define __NR_lsetxattr __NR_doesnotexist
+#define __NR_fsetxattr __NR_doesnotexist
+#define __NR_listxattr __NR_doesnotexist
+#define __NR_llistxattr __NR_doesnotexist
+#define __NR_flistxattr __NR_doesnotexist
+#define __NR_removexattr __NR_doesnotexist
+#define __NR_lremovexattr __NR_doesnotexist
+#define __NR_fremovexattr __NR_doesnotexist
+#endif
+
+#ifndef __NR_preadv
+#define __NR_preadv __NR_doesnotexist
+#endif
+#ifndef __NR_pwritev
+#define __NR_pwritev __NR_doesnotexist
+#endif
 
 //#####################################
 
@@ -269,11 +379,11 @@ struct viewinfo {
 #define LIST_SERVICE 3
 #define NAME_SERVICE 4
 #define LOCK_SERVICE 5
-#define RECURSIVE_UMVIEW   0x100
-#define UMVIEW_GETINFO     0x101
-#define UMVIEW_SETVIEWNAME 0x102
-#define UMVIEW_KILLALL     0x103
-#define UMVIEW_ATTACH      0x104
-#define UMVIEW_FSALIAS     0x105
+#define RECURSIVE_VIEWOS   0x100
+#define VIEWOS_GETINFO     0x101
+#define VIEWOS_SETVIEWNAME 0x102
+#define VIEWOS_KILLALL     0x103
+#define VIEWOS_ATTACH      0x104
+#define VIEWOS_FSALIAS     0x105
 
 #endif // _DEFS_H
