@@ -18,14 +18,14 @@
  *   with this program; if not, write to the Free Software Foundation, Inc.,
  *   51 Franklin St, Fifth Floor, Boston, MA  02110-1301 USA.
  *
- *   $Id: kmview-main.h 377 2007-08-28 20:36:16Z garden $
+ *   $Id: kmview-main.h 668 2009-05-30 15:08:31Z rd235 $
  *
  */
 
 #include <config.h>
 
 #define KMVIEW_NAME PACKAGE_NAME
-#define KMVIEW_DESC "kernel-mode View-OS"
+#define KMVIEW_DESC "kmview: kernel-module implementation of View-OS"
 #define KMVIEW_VERSION PACKAGE_VERSION
 
 #define KMVIEW_URL "http://savannah.nongnu.org/projects/view-os"
