@@ -20,7 +20,7 @@
  *   with this program; if not, write to the Free Software Foundation, Inc.,
  *   51 Franklin St, Fifth Floor, Boston, MA  02110-1301 USA.
  *
- *   $Id: kmview.c 668 2009-05-30 15:08:31Z rd235 $
+ *   $Id: kmview.c 968 2011-08-03 10:16:26Z rd235 $
  *
  */   
 #include <stdio.h>
@@ -29,13 +29,16 @@
 #include <dlfcn.h>
 #include <fcntl.h>
 #include <poll.h>
+#include <errno.h>
 #include <sys/select.h>
 #include <sys/time.h>
 #include <sys/resource.h>
 #include <stdlib.h>
 #include <assert.h>
 #include <signal.h>
+#ifdef OLDVIRSC
 #include <linux/sysctl.h>
+#endif
 #include <config.h>
 
 #ifndef _VIEWOS_KM
@@ -55,9 +58,9 @@
 #include "loginshell.h"
 
 #ifdef GDEBUG_ENABLED
-#	define OPTSTRING "+p:f:o:hvxqV:u"
+#	define OPTSTRING "+p:f:o:hvxqV:us"
 #else
-#	define OPTSTRING "+p:f:hvxqV:u"
+#	define OPTSTRING "+p:f:hvxqV:us"
 #endif
 #define KMVIEW_USER_NESTING
 
@@ -65,6 +68,8 @@ int _umview_version = 2; /* modules interface version id.
 										modules can test to be compatible with
 										um-viewos kernel*/
 unsigned int quiet = 0;
+unsigned int printk_current_level = PRINTK_STARTUP_LEVEL;
+unsigned int secure = 0;
 static char *viewname;
 
 extern int nprocs;
@@ -89,6 +94,7 @@ static void preadd(struct prelist **head,char *module)
 
 #ifdef KMVIEW_USER_NESTING
 /* virtual syscall for the underlying umview */
+#ifdef OLDVIRSC
 static long int_virnsyscall(long virscno,int n,long arg1,long arg2,long arg3,long arg4,long arg5,long arg6) {
 	struct __sysctl_args scarg;
 	long args[6]={arg1,arg2,arg3,arg4,arg5,arg6};
@@ -100,27 +106,30 @@ static long int_virnsyscall(long virscno,int n,long arg1,long arg2,long arg3,lon
 	scarg.newlen=n;
 	return native_syscall(__NR__sysctl,&scarg);
 }
+#else
+static long int_virnsyscall(long virscno,int n,long arg1,long arg2,long arg3,long arg4,long arg5,long arg6) {
+	long args[6]={arg1,arg2,arg3,arg4,arg5,arg6};
+	return native_syscall(__NR_pivot_root,NULL,n,virscno,args);
+}
+#endif
 #endif
 
 /* preload of modules */
 static int do_preload(struct prelist *head)
 {
 	if (head != NULL) {
-		void *handle;
 		int rv=do_preload(head->next);
-		handle=open_dllib(head->module);
-		if (handle==NULL) {
-			fprintf(stderr, "%s\n",dlerror());
-			return -1;
-		} else {
-			set_handle_new_service(handle,0);
+		if (add_service(head->module,0) < 0) {
+			printk("module preload %s",strerror(errno));
+			return -1 ;
+		} else
 			return rv;
-		}
 		free(head);
 	} else
 		return 0;
 }
 
+
 static void do_set_viewname(char *viewname)
 {
 	if (viewname) {
@@ -135,7 +144,7 @@ static int do_preload_recursive(struct prelist *head)
 {
 	if (head != NULL) {
 		do_preload_recursive(head->next);
-		int_virnsyscall(__NR_UM_SERVICE,3,ADD_SERVICE,0,(long)head->module,0,0,0);
+		int_virnsyscall(__NR_UM_SERVICE,3,ADD_SERVICE,(long)head->module,0,0,0,0);
 		free(head);
 		return 0;
 	} else
@@ -145,7 +154,7 @@ static int do_preload_recursive(struct prelist *head)
 static void do_set_viewname_recursive(char *viewname)
 {
 	if (viewname) {
-		int_virnsyscall(__NR_UM_SERVICE,2,UMVIEW_SETVIEWNAME,(long)viewname,0,0,0,0);
+		int_virnsyscall(__NR_UM_SERVICE,2,VIEWOS_SETVIEWNAME,(long)viewname,0,0,0,0);
 	}
 }
 #endif
@@ -181,7 +190,8 @@ static void usage(char *s)
 			"  -o file, --output file    send debug messages to file instead of stderr\n"
 #endif
 			"  -x, --nonesting           do not permit module nesting\n"
-			"  -u, --userrecursion       recursive invocation on the existing hypervisor\n",
+			"  -u, --userrecursion       recursive invocation on the existing hypervisor\n"
+			"  -s, --secure		           force permissions and capabilities\n",
 			s);
 	exit(0);
 }
@@ -197,6 +207,7 @@ static struct option long_options[] = {
 	{"help",0,0,'h'},
 	{"nonesting",0,0,'x'},
 	{"userrecursion",0,0,'u'},
+	{"secure",0,0,'s'},
 	{0,0,0,0}
 };
 
@@ -204,6 +215,7 @@ static struct option long_options[] = {
 static void load_it_again(int argc,char *argv[],int login)
 {
 	int nesting=1;
+	optind=1;
 	while (1) {
 		int c;
 		int option_index = 0;
@@ -258,6 +270,7 @@ static void kmview_recursive(int argc,char *argv[])
 		exit(1);
 	}
 
+	optind=1;
 	while (1) {
 		int c;
 		int option_index = 0;
@@ -302,10 +315,11 @@ static void kmview_recursive(int argc,char *argv[])
 static int test_recursion(int argc,char *argv[])
 {
 	int userrecursion=0;
+	optind=1;
 	while (1) {
 		int c;
 		int option_index = 0;
-		c = getopt_long(argc, argv, OPTSTRING, NULL, &option_index);
+		c = getopt_long(argc, argv, OPTSTRING, long_options, &option_index);
 		if (c == -1) break;
 		switch (c) {
 			case 'u':
@@ -321,7 +335,7 @@ static int test_recursion(int argc,char *argv[])
 static void root_process_init()
 {
 	capture_nested_init();
-	setenv("_INSIDE_UMVIEW_MODULE","",1);
+	setenv("_INSIDE_VIEWOS_MODULE","",1);
 	do_preload(prehead);
 	do_set_viewname(viewname);
 }
@@ -332,7 +346,12 @@ static void root_process_init()
 int main(int argc,char *argv[])
 {
 	char *rcfile=NULL;
-	if (argc == 1 && argv[0][0] == '-') /* login shell */
+	/*loginshell is true if this execution is driven by a login shell
+		(maybe indirectly: main restarted from /etc/viewospasswd or
+		reloaded for purelibc) */
+	int loginshell=isloginshell(argv[0]);
+	/* login shell? (directly from /etc/passwd) */
+	if (argc == 1 && argv[0][0] == '-' && argv[0][1] != '-') 
 		loginshell_view();
 	if (argc < 2) /* NO ARGS */
 	{
@@ -346,21 +365,23 @@ int main(int argc,char *argv[])
 	/* if it was setuid, return back to the user status immediately,
 	 * for safety! */
 	r_setuid(getuid());
-	/* if this is a nested invocation of umview, notify the umview monitor
-	 * and execute the process, 
-	 * try the nested invocation notifying virtual syscall, 
-	 * if it succeeded it is actually a nested invocation,
-	 * otherwise nobody is notified and the call fails*/
+	/* Check these cases only when *not* reloaded for purelibc */
+	if (strncmp(argv[0],"--kmview",8)!=0) {
+		/* if this is a nested invocation of umview, notify the umview monitor
+		 * and execute the process, 
+		 * try the nested invocation notifying virtual syscall, 
+		 * if it succeeded it is actually a nested invocation,
+		 * otherwise nobody is notified and the call fails*/
 #ifdef KMVIEW_USER_NESTING
-	if (test_recursion(argc,argv)) {
-		if (int_virnsyscall(__NR_UM_SERVICE,1,RECURSIVE_UMVIEW,0,0,0,0,0) >= 0)
-			kmview_recursive(argc,argv);	/* do not return!*/
-	}
+		if (test_recursion(argc,argv)) {
+			if (int_virnsyscall(__NR_UM_SERVICE,1,RECURSIVE_VIEWOS,0,0,0,0,0) >= 0)
+				kmview_recursive(argc,argv);	/* do not return!*/
+		}
 #endif
-	/* umview loads itself twice if there is pure_libc, to trace module 
-	 * generated syscalls, this condition manages the first call */
-	if (strncmp(argv[0],"--kmview",8)!=0)
-		load_it_again(argc,argv,isloginshell(argv[0]));	/* do not return (when purelibc and not -x)!*/
+		/* umview loads itself twice if there is pure_libc, to trace module 
+		 * generated syscalls, this condition manages the first call */
+		load_it_again(argc,argv,loginshell);	/* do not return (when purelibc and not -x)!*/
+	}
 
 	/* does this kernel provide pselect? */
 	/*has_pselect=has_pselect_test();*/
@@ -396,6 +417,9 @@ int main(int argc,char *argv[])
 			case 'q':
 				quiet = 1;
 				break;
+			case 's':
+				secure = 1;
+				break;
 #ifdef GDEBUG_ENABLED
 			case 'o': /* debugging output file redirection */ { 
 						if (optarg==NULL){
@@ -419,8 +443,14 @@ int main(int argc,char *argv[])
 	sigset_t unblockchild;
 	sigprocmask(SIG_BLOCK,NULL,&unblockchild);
 	pcb_inits(0);
+
+	if (quiet) {
+		setenv("_VIEWOS_QUIET","1",1);
+		printk_current_level = PRINTK_QUIET_LEVEL; /* warnings or errors only */
+	}
+
 	if (capture_main(argv+optind,root_process_init,rcfile) < 0) {
-		fprint2("Kmview: kernel module not loaded\n");
+		printk("Kmview: kernel module not loaded\n");
 		exit(1);
 	}
 	mp_add(kmviewfd,POLLIN,tracehand,NULL,1);
