@@ -18,13 +18,13 @@
  *   with this program; if not, write to the Free Software Foundation, Inc.,
  *   51 Franklin St, Fifth Floor, Boston, MA  02110-1301 USA.
  *
- *   $Id: scmap.h 602 2008-08-20 15:14:20Z rd235 $
+ *   $Id: scmap.h 786 2009-09-11 13:49:57Z rd235 $
  *
  */   
 #ifndef _SCMAP_H
 #define _SCMAP_H
 #include <poll.h>
-#include "services.h"
+#include "hashtab.h"
 #include "defs.h"
 
 #define VIRSYS_UMSERVICE 1
@@ -32,26 +32,26 @@
 #define __NR_msocket VIRSYS_MSOCKET
 
 /* macro for privatescno (as viewed by modules)
- * E-xtended scno */
+ * E-xtended scno (max 16384 syscall per type) */
 #if __NR_socketcall != __NR_doesnotexist
-#define ESCNO_SOCKET	0x40000000
+#define ESCNO_SOCKET	0x4000
 #else
-#define ESCNO_SOCKET	0x00000000
+#define ESCNO_SOCKET	0x0000
 #endif
-#define ESCNO_VIRSC		0x80000000
-#define ESCNO_MASK	0x3fffffff
-#define ESCNO_MAP		0xC0000000
+#define ESCNO_VIRSC		0x8000
+#define ESCNO_MASK	0x3fff
+#define ESCNO_MAP		0xC000
 
 //typedef struct service *sss;
-typedef service_t (* serfun)();
-typedef service_t serfunt();
+typedef struct ht_elem *(*htfun)();
+typedef struct ht_elem *htfunt();
 typedef long sysfunt();
 typedef long wrapinfun();
 typedef long wrapoutfun();
 typedef long wrapfun();
 /*
 typedef int wrapinfun(int sc_number,struct pcb *pc,
-		                service_t sercode, sysfun um_syscall);
+		                struct ht *hte, sysfun um_syscall);
 typedef int wrapoutfun(int sc_number,struct pcb *pc);
 */
 
@@ -67,7 +67,7 @@ struct sc_map {
 	int scno;
 	/* the choice function: this function tells the service which have to
 	 * manage the system call */
-	serfun scchoice;
+	htfun scchoice;
 	/* wrapin function: this function is called in the IN phase of the
 	 * syscall */
 	sysfun wrapin;
@@ -75,7 +75,7 @@ struct sc_map {
 	sysfun wrapout;
 	/* the choice function: this function tells the service which have to
 	 * manage the nested system call */
-	serfun nestchoice;
+	htfun nestchoice;
 	/* wrapin function: this function is called for wrapped syscalls.
 	 syscall */
 	sysfun nestwrap;
@@ -83,12 +83,26 @@ struct sc_map {
 	 * flag, the CB_R flag, etc... (look below) */
 	short flags;
 	/* number of arguments of this system call - used for some
-	 * optimizations */
-	char nargs;
+	 * optimizations (lower three bits) */
+	/* path argument (for rewriting) in three more bits
+		 bits 7,6 unused
+		 bits 5,4,3 path arg (+1 !! path is arg 0
+		 bits 2,1,0 nargs */
+	char nargx;
 	/* set of calls, for a better selection (choice fun)*/
 	unsigned char setofcall;
 };
 
+#define NARGS(X) ((int)(X & 0x7))
+#define ISPATHARG(X) ((X >> 3) & 0x7)
+#define PATHARG(X) (((X >> 3) & 0x7) - 1)
+#define PATH0 (1<<3)
+#define PATH1 (2<<3)
+#define PATH2 (3<<3)
+#define PATH3 (4<<3)
+#define PATH4 (5<<3)
+#define PATH5 (6<<3)
+
 extern struct sc_map scmap[];
 extern int scmap_scmapsize;
 extern struct sc_map virscmap[];
