@@ -19,7 +19,7 @@
  *   with this program; if not, write to the Free Software Foundation, Inc.,
  *   51 Franklin St, Fifth Floor, Boston, MA  02110-1301 USA.
  *
- *   $Id: sctab.h 620 2008-09-07 21:05:24Z rd235 $
+ *   $Id: sctab.h 866 2010-08-11 07:17:33Z rd235 $
  *
  */   
 #ifndef __SCTAB_H
@@ -29,12 +29,15 @@
 #include <sys/types.h>
 #include <unistd.h>
 #include <pthread.h>
+#include <assert.h>
 
 #include "umproc.h"
 #include "defs.h"
 #include "treepoch.h"
 
 extern int _umview_version;
+extern uid_t host_uid;
+extern gid_t host_gid;
 
 //extern pthread_key_t pcb_key; /* key to grab the current thread pcb */
 
@@ -45,7 +48,7 @@ struct pcb_fs {
 	/* filesystem informations (current working directory, root filesystem
 	 * [chroot...], and umask) */
 	char *cwd;
-	void *root;
+	char *root;
 	mode_t mask;
 };
 
@@ -59,8 +62,6 @@ extern char um_patherror[];
 
 void scdtab_init();
 
-char *um_cutdots(char *path);
-
 char *um_getpath(long laddr,struct pcb *pc);
 char *um_abspath(int dirfd, long laddr,struct pcb *pc,struct stat64 *pst,int dontfollowlink);
 
@@ -72,15 +73,34 @@ void killall(struct pcb *pc, int signo);
 void um_set_errno(struct pcb *pc,int i);
 //char *um_getcwd(struct pcb *pc,char *buf,int size);
 char *um_getroot(struct pcb *pc);
-int um_x_access(char *filename,int mode, struct pcb *pc);
-int um_x_lstat64(char *filename, struct stat64 *buf, struct pcb *pc);
+int um_x_lstat64(char *filename, struct stat64 *buf, struct pcb *pc, int isdotdot);
+/* um_x_access and um_x_readlink must follow a um_x_lstat64 */
+int um_x_access(char *filename,int mode, struct pcb *pc, struct stat64 *stbuf);
 int um_x_readlink(char *path, char *buf, size_t bufsiz, struct pcb *pc);
-epoch_t um_setepoch(epoch_t epoch);
+int um_parentwaccess(char *filename, struct pcb *pc);
+int um_xx_access(char *filename,int mode, struct pcb *pc);
+/* rewrite the path argument of a call */
+int um_x_rewritepath(struct pcb *pc, char *path, int arg, long offset);
+epoch_t um_setnestepoch(epoch_t epoch);
+int capcheck(int capability, struct pcb *pc);
+static inline int in_supgrplist(gid_t gid, struct pcb *pc)
+{
+	int i;
+	struct supgroups *grouplist=pc->grouplist;
+	assert(grouplist != NULL);
+	for (i=0;i<grouplist->size;i++)
+		if (grouplist->list[i] == gid)
+			return 1;
+	return 0;
+}
 
 struct timestamp *um_x_gettst();
 
 /* modules callbacks for extra args */
-int um_mod_getpid();
+int um_mod_getpid(void);
+void um_mod_set_hte(struct ht_elem *hte);
+struct ht_elem *um_mod_get_hte(void);
+extern void *um_mod_get_private_data(void);
 int um_mod_umoven(long addr, int len, void *_laddr);
 int um_mod_umovestr(long addr, int len, void *_laddr);
 int um_mod_ustoren(long addr, int len, void *_laddr);
