@@ -18,7 +18,7 @@
  *   with this program; if not, write to the Free Software Foundation, Inc.,
  *   51 Franklin St, Fifth Floor, Boston, MA  02110-1301 USA.
  *
- *   $Id: services.h 599 2008-08-19 22:05:33Z rd235 $
+ *   $Id: services.h 775 2009-09-01 21:15:23Z rd235 $
  *
  */   
 #ifndef __SERVICES_H
@@ -26,12 +26,12 @@
 #include "treepoch.h"
 
 typedef long (*sysfun)();
-typedef unsigned char service_t;
 
 typedef unsigned long c_set;
+#define MC_USER 1
 #define MC_CORECTLCLASS(x) ((x) << 1)
 #define MC_CORECTLOPT(x) ((x) << 6)
-#define MC_USERCTL(sercode, ctl) (1 | (sercode << 1) | (ctl << 9))
+#define MC_USERCTL(ctl) (MC_USER | (ctl << 1))
 
 #define MC_PROC			MC_CORECTLCLASS(0)
 #define MC_MODULE		MC_CORECTLCLASS(1)
@@ -40,29 +40,29 @@ typedef unsigned long c_set;
 #define MC_ADD			MC_CORECTLOPT(0)
 #define MC_REM			MC_CORECTLOPT(1)
 
-#define MC_ALLSERVICES	((1 << (sizeof(service_t) * 8)) - 1)
-
 #define MCH_SET(c, set)		*(set) |= (1 << c)
 #define MCH_CLR(c, set)		*(set) &= ~(1 << c)
 #define MCH_ISSET(c, set)	(*(set) & (1 << c))
 #define MCH_ZERO(set)		*(set) = 0;
 
-#define CHECKNOCHECK 0 //to bee or not to bee!! :-D
-#define CHECKPATH 1
-#define CHECKSOCKET 2
-#define CHECKFSTYPE 3
-#define CHECKDEVICE 4
+#define PSEUDO_CHECK  0x80
+#define CHECKMODULE   0
+#define CHECKPATH     1
+#define CHECKSOCKET   2
+#define CHECKCHRDEVICE   3
+#define CHECKBLKDEVICE   4
 #define CHECKSC 5
 #define CHECKBINFMT 6
+#define CHECKFSALIAS 7
+#define NCHECKS 8
+#define CHECKFSTYPE      (PSEUDO_CHECK | CHECKMODULE)
+#define CHECKPATHEXACT   (PSEUDO_CHECK | CHECKPATH)
+
 // for IOCTL mgmt
 #define CHECKIOCTLPARMS   0x40000000
 #define IOCTLLENMASK      0x07ffffff
 #define IOCTL_R           0x10000000
 #define IOCTL_W           0x20000000
-struct ioctl_len_req {
-	int fd;
-	int req;
-};
 	
 #define BINFMT_MODULE_ALLOC 1
 #define BINFMT_KEEP_ARG0 2
@@ -70,18 +70,23 @@ struct binfmt_req {
 	char *path;
 	char *interp;
 	char *extraarg;
+	char *buf;
 	int flags;
 };
 
+struct ht_elem;
+
 #define ERESTARTSYS 512
 
 struct service {
 	char *name;
-	service_t code;
+	char *description;
 
 	/* handle to service data. It is used by um_service.c to store
 	 * dynamic lib handle (see dlopen (3))*/
 	void *dlhandle;
+	/* destructor for ht_elem's defined by this module */
+	void (*destructor)(int type, struct ht_elem *hte);
 
 	/* Generic notification/callback function. See ../include/module.h for
 	 * details. */
@@ -93,17 +98,11 @@ struct service {
 	 */
 	c_set ctlhs;
 
-	/* choice function: returns TRUE if this path must be managed by this module
-	 * FALSE otherwise.
-	 * Nesting modules: returns the epoch of best match (0 if non found).
-	 * checkfun functions has the following args:
-	 * 	(int type, void *arg) 
-	 * 	type is defined by CHECK... constants above
-	 * if type == CHECKIOCTLPARMS
-	 *  *arg is the ioctl code
+	/* 
+	 * 	(int fd, void *req) 
 	 *  returns: the length of the field bit_or IOCTL_R/IOCTL_W if the parameter is input/output
 	 */
-	epochfun checkfun;
+	sysfun ioctlparms;
 
 	/* proactive management of select/poll system call. The module provides this function
 	 * to activate a callback when an event occurs.
@@ -130,23 +129,11 @@ struct service {
 #define UM_NONE 0xff
 #define UM_ERR 0x00
 
-int isnosys(sysfun f);
-int add_service(struct service *s);
-int set_handle_new_service(void *dlhandle,int position);
-void *get_handle_service(service_t code);
-int del_service(service_t code);
-int mov_service(service_t code, int position);
-int list_services(service_t *buf,int len);
-int name_service(service_t code,char *buf,int len);
-void lock_services();
-void invisible_services();
-void service_ctl(unsigned long type, service_t code, int skip, ...);
-service_t service_check(int type,void *arg,int setepoch);
-sysfun service_syscall(service_t code, int scno);
-sysfun service_socketcall(service_t code, int scno);
-sysfun service_virsyscall(service_t code, int scno);
-epochfun service_checkfun(service_t code);
-sysfun service_event_subscribe(service_t code);
+int add_service(char *file,int permanent);
+int del_service(char *name);
+int list_services(char *buf,int len);
+int name_service(char *name,char *buf,int len);
+void service_ctl(unsigned long type, char *sender, char *destination, ...);
 void _service_init();
 void service_addregfun(int, sysfun, sysfun);
 
