@@ -18,7 +18,7 @@
  *   with this program; if not, write to the Free Software Foundation, Inc.,
  *   51 Franklin St, Fifth Floor, Boston, MA  02110-1301 USA.
  *
- *   $Id: uid16to32.h 674 2009-06-14 18:28:57Z garden $
+ *   $Id: uid16to32.h 866 2010-08-11 07:17:33Z rd235 $
  *
  */   
 #ifndef _UID16TO32_H
@@ -57,6 +57,9 @@
 #ifndef __NR_getgroups32
 #define __NR_getgroups32 __NR_getgroups
 #endif
+#ifndef __NR_setgroups32
+#define __NR_setgroups32 __NR_setgroups
+#endif
 #ifndef __NR_getresuid32
 #define __NR_getresuid32 __NR_getresuid
 #endif
