@@ -19,7 +19,7 @@
  *   with this program; if not, write to the Free Software Foundation, Inc.,
  *   51 Franklin St, Fifth Floor, Boston, MA  02110-1301 USA.
  *
- *   $Id: um_exec.c 603 2008-08-20 15:30:43Z rd235 $
+ *   $Id: um_exec.c 968 2011-08-03 10:16:26Z rd235 $
  *
  */   
 #include <assert.h>
@@ -32,6 +32,7 @@
 #include <sys/uio.h>
 #include <sys/types.h>
 #include <sys/time.h>
+#include <sys/mount.h>
 #include <asm/ptrace.h>
 #include <asm/unistd.h>
 #include <linux/net.h>
@@ -45,90 +46,77 @@
 #include "defs.h"
 #include "gdebug.h"
 #include "umproc.h"
-#include "services.h"
+#include "hashtab.h"
 #include "um_services.h"
 #include "sctab.h"
 #include "scmap.h"
 #include "utils.h"
 #include "canonicalize.h"
 
-#define SCRIPTBUFLEN 128
-#define UM_SCRIPT UM_ERR /*we use UM_ERR to say UM_SCRIPT */
+#define BINFMTBUFLEN 128
+#define HT_SCRIPT ((struct ht_elem *) 1)
 #define STDINTERP "/bin/bash"
 
 /* filecopy creates a copy of the executable inside the tmp file dir */
-static int filecopy(service_t sercode,const char *from, const char *to)
+static int filecopy(struct ht_elem *hte,const char *from, const char *to)
 {
 	char buf[BUFSIZ];
 	int fdf,fdt;
 	int n;
-	if ((fdf=service_syscall(sercode,uscno(__NR_open))(from,O_RDONLY,0)) < 0)
+	/* NO need for hte search. from is the path so hte and private data
+		 is already set for modules */
+	if ((fdf=ht_syscall(hte,uscno(__NR_open))(from,O_RDONLY,0)) < 0)
 		return -errno;
 	if ((fdt=open(to,O_CREAT|O_TRUNC|O_WRONLY,0600)) < 0)
 		return -errno;
-	while ((n=service_syscall(sercode,uscno(__NR_read))(fdf,buf,BUFSIZ)) > 0)
+	while ((n=ht_syscall(hte,uscno(__NR_read))(fdf,buf,BUFSIZ)) > 0)
 		r_write (fdt,buf,n);
-	service_syscall(sercode,uscno(__NR_close))(fdf);
+	ht_syscall(hte,uscno(__NR_close))(fdf);
 	fchmod (fdt,0700); /* permissions? */
 	close (fdt);
 	return 0;
 }
 
 /* is the executable a script? */
-static int checkscript(service_t sercode,struct binfmt_req *req,char *scriptbuf)
+static struct ht_elem *checkscript(struct ht_elem *hte,struct binfmt_req *req)
 {
-	int fd,n;
-	if (sercode == UM_NONE) {
-		if ((fd=open(req->path,O_RDONLY,0)) < 0)
-			return UM_NONE;
-		n=read(fd,scriptbuf,SCRIPTBUFLEN-1);
-		close(fd);
-	} else {
-		if ((fd=service_syscall(sercode,uscno(__NR_open))(req->path,O_RDONLY,0)) < 0)
-			return UM_NONE;
-		n=service_syscall(sercode,uscno(__NR_read))(fd,scriptbuf,SCRIPTBUFLEN-1);
-		service_syscall(sercode,uscno(__NR_close))(fd);
-	}
-	if (n>1) {
-		/* this should include ELF, COFF and a.out */
-		if (scriptbuf[0] < '\n' || scriptbuf[0] == '\177')
-			return UM_NONE;
-		else if (scriptbuf[0]=='#' && scriptbuf[1]=='!') {
-			/* parse the first line */
-			char *s=scriptbuf+2;
-			scriptbuf[n]=0;
-			/* skip leading spaces */
-			while(*s == ' ' || *s == '\t')
-				s++;
-			/* the first non blank is the interpreter */
-			req->interp=s; 
-			while(*s != ' ' && *s != '\t' && *s != '\n' && *s!=0)
-				s++;
-			if (*s == 0 || *s=='\n') 
-				*s=0;
-			else {
-				*s++ = 0;
-				while(*s == ' ' || *s == '\t')
-					*s++ = 0;
-				req->extraarg=s;
-				while(*s != '\n' && *s!=0)
-					s++;
-				while(*(s-1)==' ' || *(s-1)=='\t')
-					s--;
-				*s=0;
-				if (*(req->extraarg)==0)
-					req->extraarg=NULL;
-			}
-			if (*(req->interp)==0)
-				req->interp=STDINTERP;
-			return UM_SCRIPT;
-		}
+	char *scriptbuf=req->buf;
+	/* this should include ELF, COFF and a.out */
+	if (scriptbuf[0] < '\n' || scriptbuf[0] == '\177')
+		return NULL;
+	else if (scriptbuf[0]=='#' && scriptbuf[1]=='!') {
+		/* parse the first line */
+		char *s=scriptbuf+2;
+		/* skip leading spaces */
+		while(*s == ' ' || *s == '\t')
+			s++;
+		/* the first non blank is the interpreter */
+		req->interp=s; 
+		while(*s != ' ' && *s != '\t' && *s != '\n' && *s!=0)
+			s++;
+		if (*s == 0 || *s=='\n') 
+			*s=0;
 		else {
-			/* heuristics here */
-			return UM_NONE;
+			*s++ = 0;
+			while(*s == ' ' || *s == '\t')
+				*s++ = 0;
+			req->extraarg=s;
+			while(*s != '\n' && *s!=0)
+				s++;
+			while(*(s-1)==' ' || *(s-1)=='\t')
+				s--;
+			*s=0;
+			if (*(req->extraarg)==0)
+				req->extraarg=NULL;
 		}
+		if (*(req->interp)==0)
+			req->interp=STDINTERP;
+		return HT_SCRIPT;
+	}
+	else {
+		/* heuristics here */
+		return NULL;
 	}
-	return UM_NONE;
 }
 
 /* getparms (argv) from the user space */
@@ -140,14 +128,20 @@ static char **getparms(struct pcb *pc,long laddr) {
 	int n=0;
 	int i;
 	do {
-		int rv;
 		if (n >= size) {
 			size+=CHUNKSIZE;
 			paddr=realloc(paddr,size*sizeof(long));
 			assert(paddr);
 		}
-		rv=umoven(pc,laddr,sizeof(char *),&(paddr[n]));
-		assert(rv>=0);
+#ifdef NDEBUG
+		umoven(pc,laddr,sizeof(char *),&(paddr[n]));
+#else
+		{
+			int rv;
+			rv=umoven(pc,laddr,sizeof(char *),&(paddr[n]));
+			assert(rv>=0);
+		}
+#endif
 		laddr+= sizeof(char *);
 		n++;
 	} while (paddr[n-1] != 0);
@@ -178,56 +172,62 @@ static void freeparms(char **parms)
 /*
 static void printparms(char *what,char **parms)
 {
-	fprint2("%s\n",what);
+	printk("%s\n",what);
 	while (*parms != 0) {
-		fprint2("--> %s\n",*parms);
+		printk("--> %s\n",*parms);
 		parms++;
 	}
 }
 */
 
-static int is_regular(char *path,struct pcb *pc)
-{
-	char newpath[PATH_MAX];
-	struct stat64 st;
-	um_realpath(path,"/",newpath,&st,0,pc);
-	if (S_ISREG(st.st_mode))
-		return 1;
-	else {
-		if (st.st_mode==0) 
-			pc->erno=ENOENT;
-		else
-			pc->erno=EACCES;
-		return 0;
-	}
-}
-
 #define UMBINWRAP LIBEXECDIR "/umbinwrap"
-/* WIP XXX try to run scripts and interpreters without UMBINWRAP */
-#undef NOUMBINWRAP
 /* wrap_in: execve handling */
 int wrap_in_execve(int sc_number,struct pcb *pc,
-		service_t sercode,sysfun um_syscall)
+		struct ht_elem *hte,sysfun um_syscall)
 {
-	struct binfmt_req req={(char *)pc->path,NULL,NULL,0};
-	char scriptbuf[SCRIPTBUFLEN];
-	epoch_t nestepoch=um_setepoch(0);
-	service_t binfmtser;
-	if (um_x_access(req.path,X_OK,pc)!=0) {
+	char buf[BINFMTBUFLEN+1];
+	struct binfmt_req req={(char *)pc->path,NULL,NULL,buf,0};
+	epoch_t nestepoch=um_setnestepoch(0);
+	struct ht_elem *binfmtht;
+	if (um_xx_access(req.path,X_OK,pc)!=0) {
 		pc->erno=errno;
 		pc->retval=-1;
 		return SC_FAKE;
 	}
+	/* management of set[ug]id executables */
+	if (!secure || !(ht_get_mountflags(pc->hte) & MS_NOSUID)) {
+		if (pc->pathstat.st_mode & S_ISUID) {
+			pc->suid=pc->euid;
+			pc->euid=pc->fsuid=pc->pathstat.st_uid;
+		} else if (pc->ruid == pc->euid)
+			pc->suid=pc->ruid;
+		if (pc->pathstat.st_mode & S_ISGID) {
+			pc->sgid=pc->egid;
+			pc->egid=pc->fsgid=pc->pathstat.st_gid;
+		} else if (pc->rgid == pc->egid)
+			pc->sgid=pc->rgid;
+	}
+	if (!secure && (strcmp(pc->path,"/bin/mount") == 0 || 
+		strcmp(pc->path,"/bin/umount") == 0)) {
+		pc->suid=pc->euid;
+		pc->ruid=pc->suid=0;
+	}
 	/* The epoch should be just after the mount 
 	 * which generated the executable */
-	um_setepoch(nestepoch+1);
-	binfmtser=checkscript(sercode,&req,scriptbuf);
-	if (binfmtser == UM_NONE)
-		binfmtser=service_check(CHECKBINFMT,&req,0);
-	//fprint2("wrap_in_execve %x |%s| |%s|\n",binfmtser,req.interp,req.extraarg);
-	um_setepoch(nestepoch);
+	um_setnestepoch(nestepoch+1);
+	memset(buf,0,BINFMTBUFLEN+1);
+	int fd=open(req.path,O_RDONLY);
+	if (fd >= 0) {
+		read(fd, buf, BINFMTBUFLEN);
+		close(fd);
+	}
+	binfmtht=checkscript(hte,&req);
+	if (binfmtht == NULL) 
+		binfmtht=ht_check(CHECKBINFMT,&req,NULL,0);
+	//printk("wrap_in_execve %s |%s| |%s|\n",ht_get_servicename(binfmtht),req.interp,req.extraarg);
+	um_setnestepoch(nestepoch);
 	/* is there a binfmt service for this executable? */
-	if (binfmtser != UM_NONE) {
+	if (binfmtht != NULL) {
 		char *umbinfmtarg0;
 		int sep;
 		long largv=pc->sysargs[1];
@@ -237,25 +237,21 @@ int wrap_in_execve(int sc_number,struct pcb *pc,
 		int filenamelen;
 		int arg0len;
 		long sp=getsp(pc);
+		char *chrootpath=pc->path;
 		if (*(req.interp) != '/') { /* full pathname required */
 			pc->erno=ENOENT;
 			pc->retval=-1;
 			return SC_FAKE;
 		}
-#ifndef NOUMBINWRAP
-		if (!is_regular(req.interp,pc)) {
-			pc->retval=-1;
-			return SC_FAKE;
+		/* strip the root path when running in a chroot environment */
+		if (pc->fdfs->root[1] != 0) {
+			int len=strlen(pc->fdfs->root);
+			if (strncmp(chrootpath,pc->fdfs->root,len)==0)
+				chrootpath+=len;
 		}
-		if (um_x_access(req.interp,X_OK,pc)!=0) {
-			pc->erno=errno;
-			pc->retval=-1;
-			return SC_FAKE;
-		}
-#endif
 		/* create the argv for the wrapper! */
 		rv=umoven(pc,largv,sizeof(char *),&(larg0));
-		//fprint2("%s %d %ld %ld rv=%d\n",pc->path,getpc(pc),largv,larg0,rv); 
+		//printk("%s %d %ld %ld rv=%d\n",chrootpath,getpc(pc),largv,larg0,rv); 
 		/* XXX this is a workaround. strace has the same error!
 		 * exec seems to cause an extra prace in a strange address space
 		 * to be solved (maybe using PTRACE OPTIONS!) */
@@ -268,27 +264,26 @@ int wrap_in_execve(int sc_number,struct pcb *pc,
 			oldarg0[0]=0;
 		/* search for an unused char to act as arg separator */
 		for (sep=1;sep<255 && 
-				(strchr((char *)pc->path,sep)!=NULL ||
+				(strchr((char *)chrootpath,sep)!=NULL ||
 				 strchr(req.interp,sep)!=NULL ||
 				 strchr(oldarg0,sep)!=NULL);
 				sep++)
 			;
 		if (req.extraarg==NULL)
 			req.extraarg="";
-#ifdef NOUMBINWRAP
-#else
+#ifndef NOUMBINWRAP
 		/* collapse all the args in only one arg */
 		if (req.flags & BINFMT_KEEP_ARG0) 
 			asprintf(&umbinfmtarg0,"%c%s%c%s%c%s%c%s",
 					sep,req.interp,
 					sep,req.extraarg,
-					sep,(char *)pc->path,
+					sep,(char *)chrootpath,
 					sep,oldarg0);
 		else 
 			asprintf(&umbinfmtarg0,"%c%s%c%s%c%s",
 					sep,req.interp,
 					sep,req.extraarg,
-					sep,(char *)pc->path);
+					sep,(char *)chrootpath);
 		filenamelen=WORDALIGN(strlen(UMBINWRAP));
 		arg0len=WORDALIGN(strlen(umbinfmtarg0));
 		pc->retval=0;
@@ -297,7 +292,7 @@ int wrap_in_execve(int sc_number,struct pcb *pc,
 		larg0=sp-filenamelen-arg0len;
 		ustoren(pc,larg0,arg0len,umbinfmtarg0);
 		ustoren(pc,largv,sizeof(char *),&larg0);
-		//fprint2("%s %s\n",UMBINWRAP,umbinfmtarg0);
+		//printk("%s %s\n",UMBINWRAP,umbinfmtarg0);
 		/* exec the wrapper instead of the executable! */
 		free(umbinfmtarg0);
 #endif
@@ -305,10 +300,14 @@ int wrap_in_execve(int sc_number,struct pcb *pc,
 			free(req.interp);
 		return SC_CALLONXIT;
 	}
-	else if (sercode != UM_NONE) {
+	else if (hte != NULL) {
 		pc->retval=ERESTARTSYS;
+		if (secure && (ht_get_mountflags(hte) & MS_NOEXEC)) {
+			pc->retval=-1;
+			pc->erno=EACCES;
+		}
 		/* does the module define a semantics for execve? */
-		if (!isnosys(um_syscall)) {
+		else if (!isnosys(um_syscall)) {
 			long largv=pc->sysargs[1];
 			long lenv=pc->sysargs[2];
 			char **argv=getparms(pc,largv);
@@ -325,18 +324,14 @@ int wrap_in_execve(int sc_number,struct pcb *pc,
 		 * to require the real execve */
 		if (pc->retval==ERESTARTSYS){
 			char *filename=strdup(um_proc_tmpname());
-			//fprint2("wrap_in_execve! %s %p %d\n",(char *)pc->path,um_syscall,isnosys(um_syscall));
+			//printk("wrap_in_execve! %s %p %d\n",(char *)pc->path,um_syscall,isnosys(um_syscall));
 
 			/* copy the file and change the first arg of execve to 
 			 * address the copy */
-			if ((pc->retval=filecopy(sercode,pc->path,filename))>=0) {
-				long sp=getsp(pc);
-				int filenamelen=WORDALIGN(strlen(filename));
-				pc->retval=0;
+			if ((pc->retval=filecopy(hte,pc->path,filename))>=0) {
+				um_x_rewritepath(pc,filename,0,0);
 				/* remember to clean up the copy as soon as possible */
 				pc->tmpfile2unlink_n_free=filename;
-				ustoren(pc,sp-filenamelen,filenamelen,filename);
-				pc->sysargs[0]=sp-filenamelen;
 				return SC_CALLONXIT;
 			} else {
 				/* something went wrong during the copy */
@@ -348,16 +343,22 @@ int wrap_in_execve(int sc_number,struct pcb *pc,
 		} else 
 			return SC_FAKE;
 	} else 
-		return STD_BEHAVIOR;
+		if (__builtin_expect(pc->needs_path_rewrite,0)) {
+			um_x_rewritepath(pc,pc->path,0,0);
+			return SC_CALLONXIT;
+		} else
+			return STD_BEHAVIOR;
 }
 
 
 int wrap_out_execve(int sc_number,struct pcb *pc) 
 { 
 	/* If this function is executed it means that something went wrong! */
-	//fprint2("wrap_out_execve %d\n",pc->retval);
+	//printk("wrap_out_execve %d\n",pc->retval);
 	/* The tmp file gets automagically deleted (see sctab.c) */
 	if (pc->retval < 0) {
+		pc->euid=pc->fsuid=pc->suid;
+		pc->egid=pc->fsgid=pc->sgid;
 		putrv(pc->retval,pc);
 		puterrno(pc->erno,pc);
 		return SC_MODICALL;
