@@ -18,14 +18,14 @@
  *   with this program; if not, write to the Free Software Foundation, Inc.,
  *   51 Franklin St, Fifth Floor, Boston, MA  02110-1301 USA.
  *
- *   $Id: um_services.h 669 2009-05-30 16:24:08Z rd235 $
+ *   $Id: um_services.h 775 2009-09-01 21:15:23Z rd235 $
  *
  */   
 #ifndef _UM_SERVICES_H
 #define _UM_SERVICES_H
 
-char *fs_alias(char *fsalias);
+char *get_alias(int type,char *fsalias);
 
-void *open_dllib(char *name);
+//void *open_dllib(char *name);
 
 #endif
