@@ -18,7 +18,7 @@
  *   with this program; if not, write to the Free Software Foundation, Inc.,
  *   51 Franklin St, Fifth Floor, Boston, MA  02110-1301 USA.
  *
- *   $Id: um_socket.c 655 2009-04-28 18:37:08Z rd235 $
+ *   $Id: um_socket.c 994 2011-08-17 14:50:25Z rd235 $
  *
  */   
 #include <assert.h>
@@ -41,6 +41,7 @@
 #include "defs.h"
 #include "umproc.h"
 #include "services.h"
+#include "hashtab.h"
 #include "um_services.h"
 #include "sctab.h"
 #include "scmap.h"
@@ -51,7 +52,7 @@
 /* SOCKET & MSOCKET call management (IN) */
 
 int wrap_in_msocket(int sc_number,struct pcb *pc,
-		service_t sercode, sysfun um_syscall)
+		struct ht_elem *hte, sysfun um_syscall)
 {
 	/* path = pc->sysarg[0] = pc->path */
 	int domain  =pc->sysargs[1];
@@ -59,7 +60,7 @@ int wrap_in_msocket(int sc_number,struct pcb *pc,
 	int protocol=pc->sysargs[3];
 	/* msocket is ALWAYS called: msocket(NULL...) calls must be converted
 	 * into socket(...) syscalls */
-	if (sercode != UM_NONE) {
+	if (hte != NULL) {
 		if (type == SOCK_DEFAULT) {
 			if (pc->path == NULL) {
 				pc->retval = -1;
@@ -68,7 +69,7 @@ int wrap_in_msocket(int sc_number,struct pcb *pc,
 				if ((pc->retval = um_syscall(pc->path,domain,type,protocol)) < 0) {
 					pc->erno = errno;
 				}
-				//fprint2("SOCK_DEFAULT %s %d\n",pc->path,domain);
+				//printk("SOCK_DEFAULT %s %d\n",pc->path,domain);
 			}
 			return SC_FAKE;
 		} else {
@@ -78,9 +79,9 @@ int wrap_in_msocket(int sc_number,struct pcb *pc,
 					 * modules implementing only "socket". 
 					 * the code reaches this case only from wrap_in_socket */
 #if (__NR_socketcall != __NR_doesnotexist)
-					um_syscall=service_socketcall(sercode,SYS_SOCKET);
+					um_syscall=ht_socketcall(hte,SYS_SOCKET);
 #else
-					um_syscall=service_syscall(sercode,uscno(__NR_socket));
+					um_syscall=ht_syscall(hte,uscno(__NR_socket));
 #endif
 					if ((pc->retval = um_syscall(domain,type,protocol)) < 0) {
 						pc->erno = errno;
@@ -90,17 +91,9 @@ int wrap_in_msocket(int sc_number,struct pcb *pc,
 			}
 			/* create the comm fifo with the user process */
 			if (pc->retval >= 0 &&
-					(pc->retval=lfd_open(sercode,pc->retval,NULL,O_RDWR,0)) >= 0) {
-				char *filename=lfd_getfilename(pc->retval);
-				int filenamelen=WORDALIGN(strlen(filename));
-				long sp=getsp(pc);
-				/* change the system call arguments (open the fifo) */
-				/* could give some trouble if a process tests the
-				 * "type" of the file connected to the fd and the
-				 * module does not give the right answer!*/
-				ustoren(pc,sp-filenamelen,filenamelen,filename); /*socket?*/
+					(pc->retval=lfd_open(hte,pc->retval,NULL,O_RDWR,0)) >= 0) {
+				um_x_rewritepath(pc,lfd_getfilename(pc->retval),0,0);
 				putscno(__NR_open,pc);
-				pc->sysargs[0]=sp-filenamelen;
 				pc->sysargs[1]=O_RDONLY;
 				return SC_CALLONXIT;
 			} else
@@ -139,21 +132,21 @@ int wrap_in_msocket(int sc_number,struct pcb *pc,
 }
 
 int wrap_in_socket(int sc_number,struct pcb *pc,
-		service_t sercode, sysfun um_syscall)
+		struct ht_elem *hte, sysfun um_syscall)
 {
 	pc->sysargs[3]=pc->sysargs[2];
 	pc->sysargs[2]=pc->sysargs[1];
 	pc->sysargs[1]=pc->sysargs[0];
 	pc->sysargs[0]=umNULL;
 	pc->path=NULL;
-	return wrap_in_msocket(__NR_msocket,pc,sercode,
-			service_virsyscall(sercode,VIRSYS_MSOCKET));
+	return wrap_in_msocket(__NR_msocket,pc,hte,
+			ht_virsyscall(hte,VIRSYS_MSOCKET));
 }
 
 #define MAX_SOCKLEN 1024
 /* accept creates a new fd! */
 int wrap_in_accept(int sc_number,struct pcb *pc,
-		service_t sercode, sysfun um_syscall)
+		struct ht_elem *hte, sysfun um_syscall)
 {
 	int sfd=fd2sfd(pc->fds,pc->sysargs[0]);
 	/* the virtual file does not exist */
@@ -164,37 +157,41 @@ int wrap_in_accept(int sc_number,struct pcb *pc,
 		/* get the system call args */
 		long sock_plen=pc->sysargs[2];
 		int sock_len;
-		if (sock_plen != umNULL)
+		if (sock_plen != umNULL) {
 			umoven(pc,sock_plen,4,&sock_len);
-		/* safety check for sock */
-		if (sock_len == 0 || sock_len > MAX_SOCKLEN) {
-			pc->retval= -1;
-			pc->erno= EINVAL;
-		} else {
-			long sock_addr=pc->sysargs[1];
-			char *sock=(char *)alloca(sock_len);
-			/* get the sock_addr */
+			/* safety check for sock */
+			if (sock_len == 0 || sock_len > MAX_SOCKLEN) {
+				pc->retval= -1;
+				pc->erno= EINVAL;
+			} else {
+				long sock_addr=pc->sysargs[1];
+				char *sock;
+				if (__builtin_expect((sock_len > MAX_SOCKET_NAME),0)) 
+					sock_len=MAX_SOCKET_NAME;
+				sock=(char *)alloca(sock_len);
+				/* get the sock_addr */
 				umoven(pc,sock_addr,sock_len,sock);
 				/* virtual syscall */
-			if ((pc->retval = um_syscall(sfd,sock,&sock_len)) < 0)
-				pc->erno=errno;
-			else {
-			/* store the results (if the call was successful) */
-				if (sock_addr != umNULL)
-					ustoren(pc,sock_addr,sock_len,sock);
-				if (sock_plen != umNULL)
-					umoven(pc,sock_plen,4,&sock_len);
+				if ((pc->retval = um_syscall(sfd,sock,&sock_len)) < 0)
+					pc->erno=errno;
+				else {
+					/* store the results (if the call was successful) */
+					if (sock_addr != umNULL)
+						ustoren(pc,sock_addr,sock_len,sock);
+					if (sock_plen != umNULL)
+						umoven(pc,sock_plen,4,&sock_len);
+				}
 			}
+		} else {
+			if ((pc->retval = um_syscall(sfd,NULL,NULL)) < 0)
+				pc->erno=errno;
 		}
+
 		/* open the new fifo, (accept creates a new fd) */
 		if (pc->retval >= 0 && 
-				(pc->retval=lfd_open(sercode,pc->retval,NULL,O_RDWR,0)) >= 0) {
-			char *filename=lfd_getfilename(pc->retval);
-			int filenamelen=WORDALIGN(strlen(filename));
-			int sp=getsp(pc);
-			ustorestr(pc,sp-filenamelen,filenamelen,filename); /*socket?*/
+				(pc->retval=lfd_open(hte,pc->retval,NULL,O_RDWR,0)) >= 0) {
+			um_x_rewritepath(pc,lfd_getfilename(pc->retval),0,0);
 			putscno(__NR_open,pc);
-			pc->sysargs[0]=sp-filenamelen;
 			pc->sysargs[1]=O_RDONLY;
 			return SC_CALLONXIT;
 		} else
@@ -206,7 +203,7 @@ int wrap_in_accept(int sc_number,struct pcb *pc,
 /* SOCKET & MSOCKET & ACCEPT wrap out */
 int wrap_out_socket(int sc_number,struct pcb *pc) {
 	/*int lerno=errno;*/
-	//fprint2("wrap_out_socket %d %d\n",pc->behavior,SC_FAKE);
+	//printk("wrap_out_socket %d %d\n",pc->behavior,SC_FAKE);
 	/* if everything was okay for the virtual call */
 	if (pc->behavior==SC_CALLONXIT && pc->retval >= 0) {
 		int fd=getrv(pc);	
@@ -224,7 +221,7 @@ int wrap_out_socket(int sc_number,struct pcb *pc) {
 		} else {
 			putrv(pc->retval,pc);
 			puterrno(pc->erno,pc);
-			//fprint2("wrap_out_socket!!\n");
+			//printk("wrap_out_socket!!\n");
 			lfd_close(pc->retval);
 		}
 	} else {
@@ -235,7 +232,7 @@ int wrap_out_socket(int sc_number,struct pcb *pc) {
 }
 
 int wrap_in_bind_connect(int sc_number,struct pcb *pc,
-		service_t sercode, sysfun um_syscall)
+		struct ht_elem *hte, sysfun um_syscall)
 {
 	int sfd=fd2sfd(pc->fds,pc->sysargs[0]);
 	if (sfd < 0) {
@@ -244,7 +241,10 @@ int wrap_in_bind_connect(int sc_number,struct pcb *pc,
 	} else {
 		long sock_addr=pc->sysargs[1];
 		long sock_len=pc->sysargs[2];
-		char *sock=(char *)alloca(sock_len);
+		char *sock;
+		if (__builtin_expect((sock_len > MAX_SOCKET_NAME),0)) 
+			sock_len=MAX_SOCKET_NAME;
+		sock=(char *)alloca(sock_len);
 		umoven(pc,sock_addr,sock_len,sock);
 		if ((pc->retval = um_syscall(sfd,sock,sock_len)) < 0)
 			pc->erno=errno;
@@ -253,7 +253,7 @@ int wrap_in_bind_connect(int sc_number,struct pcb *pc,
 }
 
 int wrap_in_listen(int sc_number,struct pcb *pc,
-		service_t sercode, sysfun um_syscall)
+		struct ht_elem *hte, sysfun um_syscall)
 {
 	int sfd=fd2sfd(pc->fds,pc->sysargs[0]);
 	if (sfd < 0) {
@@ -268,7 +268,7 @@ int wrap_in_listen(int sc_number,struct pcb *pc,
 }
 
 int wrap_in_getsock(int sc_number,struct pcb *pc,
-		service_t sercode, sysfun um_syscall)
+		struct ht_elem *hte, sysfun um_syscall)
 {
 	int sfd=fd2sfd(pc->fds,pc->sysargs[0]);
 	if (sfd < 0) {
@@ -280,7 +280,10 @@ int wrap_in_getsock(int sc_number,struct pcb *pc,
 		int sock_len;
 		if (sock_plen != umNULL)
 			umoven(pc,sock_plen,4,&sock_len);
-		char *sock=(char *)alloca(sock_len);
+		char *sock;
+		if (__builtin_expect((sock_len > MAX_SOCKET_NAME),0)) 
+			sock_len=MAX_SOCKET_NAME;
+		sock=(char *)alloca(sock_len);
 		umoven(pc,sock_addr,sock_len,sock);
 		if ((pc->retval = um_syscall(sfd,sock,&sock_len)) < 0)
 			pc->erno=errno;
@@ -295,7 +298,7 @@ int wrap_in_getsock(int sc_number,struct pcb *pc,
 }
 
 int wrap_in_send(int sc_number,struct pcb *pc,
-		service_t sercode, sysfun um_syscall)
+		struct ht_elem *hte, sysfun um_syscall)
 {
 	int sfd=fd2sfd(pc->fds,pc->sysargs[0]);
 	if (sfd < 0) {
@@ -306,7 +309,7 @@ int wrap_in_send(int sc_number,struct pcb *pc,
 		long buf=pc->sysargs[1];
 		int len=pc->sysargs[2];
 		int flags=pc->sysargs[3];
-		char *lbuf=(char *)alloca(len); 
+		char *lbuf=(char *)lalloca(len); 
 #ifdef SNDRCVMSGUNIFY
 		struct iovec iov = {lbuf,len};
 		struct msghdr msg = {
@@ -326,12 +329,13 @@ int wrap_in_send(int sc_number,struct pcb *pc,
 		if ((pc->retval=um_syscall(sfd,lbuf,len,flags)) < 0)
 			pc->erno=errno;
 #endif
+		lfree(lbuf,len);
 	}
 	return SC_FAKE;
 }
 
 int wrap_in_recv(int sc_number,struct pcb *pc,
-		service_t sercode, sysfun um_syscall)
+		struct ht_elem *hte, sysfun um_syscall)
 {
 	int sfd=fd2sfd(pc->fds,pc->sysargs[0]);
 	if (sfd < 0) {
@@ -341,7 +345,7 @@ int wrap_in_recv(int sc_number,struct pcb *pc,
 		long buf=pc->sysargs[1];
 		int len=pc->sysargs[2];
 		int flags=pc->sysargs[3];
-		char *lbuf=(char *)alloca(len);
+		char *lbuf=(char *)lalloca(len);
 #ifdef SNDRCVMSGUNIFY
 		struct iovec iov = {lbuf,len};
 		struct msghdr msg = {
@@ -360,12 +364,13 @@ int wrap_in_recv(int sc_number,struct pcb *pc,
 #endif
 		if (pc->retval > 0)
 			ustoren(pc,buf,pc->retval,lbuf);
+		lfree(lbuf,len);
 	}
 	return SC_FAKE;
 }
 
 int wrap_in_sendto(int sc_number,struct pcb *pc,
-		service_t sercode, sysfun um_syscall)
+		struct ht_elem *hte, sysfun um_syscall)
 {
 	int sfd=fd2sfd(pc->fds,pc->sysargs[0]);
 	if (sfd < 0) {
@@ -377,7 +382,7 @@ int wrap_in_sendto(int sc_number,struct pcb *pc,
 		int flags=pc->sysargs[3];
 		long pto=pc->sysargs[4];
 		int tolen=pc->sysargs[5];
-		char *lbuf=(char *)alloca(len); 
+		char *lbuf=(char *)lalloca(len); 
 		char *tosock=NULL;
 #ifdef SNDRCVMSGUNIFY
 		struct iovec iov = {lbuf,len};
@@ -392,6 +397,8 @@ int wrap_in_sendto(int sc_number,struct pcb *pc,
 #endif
 		umoven(pc,buf,len,lbuf);
 		if (pto != umNULL) {
+			if (__builtin_expect((tolen > MAX_SOCKET_NAME),0)) 
+				tolen=MAX_SOCKET_NAME;
 			tosock=alloca(tolen);
 			umoven(pc,pto,tolen,tosock);
 #ifdef SNDRCVMSGUNIFY
@@ -406,12 +413,13 @@ int wrap_in_sendto(int sc_number,struct pcb *pc,
 		if ((pc->retval=um_syscall(sfd,lbuf,len,flags,tosock,tolen)) < 0)
 			pc->erno=errno;
 #endif
+		lfree(lbuf,len);
 	}
 	return SC_FAKE;
 }
 
 int wrap_in_recvfrom(int sc_number,struct pcb *pc,
-		service_t sercode, sysfun um_syscall)
+		struct ht_elem *hte, sysfun um_syscall)
 {
 	int sfd=fd2sfd(pc->fds,pc->sysargs[0]);
 	if (sfd < 0) {
@@ -424,7 +432,7 @@ int wrap_in_recvfrom(int sc_number,struct pcb *pc,
 		long pfrom=pc->sysargs[4];
 		long pfromlen=pc->sysargs[5];
 		int fromlen=0;
-		char *lbuf=(char *)alloca(len);
+		char *lbuf=(char *)lalloca(len);
 		char *fromsock=NULL;
 #ifdef SNDRCVMSGUNIFY
 		struct iovec iov = {lbuf,len};
@@ -440,6 +448,8 @@ int wrap_in_recvfrom(int sc_number,struct pcb *pc,
 		if (pfromlen != umNULL) {
 			umoven(pc,pfromlen,4,(char *)&fromlen);
 			if (pfrom != umNULL && fromlen != 0) {
+				if (__builtin_expect((fromlen > MAX_SOCKET_NAME),0)) 
+					fromlen=MAX_SOCKET_NAME;
 				fromsock=alloca(fromlen);
 				umoven(pc,pfrom,fromlen,fromsock);
 			}
@@ -462,12 +472,13 @@ int wrap_in_recvfrom(int sc_number,struct pcb *pc,
 			if (pfromlen != umNULL)
 				ustoren(pc,pfromlen,4,&fromlen);
 		}
+		lfree(lbuf,len);
 	}
 	return SC_FAKE;
 }
 
 int wrap_in_shutdown(int sc_number,struct pcb *pc,
-		service_t sercode, sysfun um_syscall)
+		struct ht_elem *hte, sysfun um_syscall)
 {
 	int sfd=fd2sfd(pc->fds,pc->sysargs[0]);
 	if (sfd < 0) {
@@ -482,7 +493,7 @@ int wrap_in_shutdown(int sc_number,struct pcb *pc,
 }
 
 int wrap_in_getsockopt(int sc_number,struct pcb *pc,
-		service_t sercode, sysfun um_syscall)
+		struct ht_elem *hte, sysfun um_syscall)
 {
 	int sfd=fd2sfd(pc->fds,pc->sysargs[0]);
 	if (sfd < 0) {
@@ -497,6 +508,8 @@ int wrap_in_getsockopt(int sc_number,struct pcb *pc,
 		void *optval;
 		if (poptlen != umNULL) {
 			umoven(pc,poptlen,4,(char *)&optlen);
+			if (__builtin_expect((optlen > MAX_SOCKOPT_LEN),0))
+				optlen=MAX_SOCKOPT_LEN;
 			optval=(optlen > 0)?alloca(optlen):NULL;
 		} else {
 			optlen=0;
@@ -513,7 +526,7 @@ int wrap_in_getsockopt(int sc_number,struct pcb *pc,
 }
 
 int wrap_in_setsockopt(int sc_number,struct pcb *pc,
-		service_t sercode, sysfun um_syscall)
+		struct ht_elem *hte, sysfun um_syscall)
 {
 	int sfd=fd2sfd(pc->fds,pc->sysargs[0]);
 	if (sfd < 0) {
@@ -525,8 +538,10 @@ int wrap_in_setsockopt(int sc_number,struct pcb *pc,
 		long poptval=pc->sysargs[3];
 		int optlen=pc->sysargs[4];
 		void *optval;
-		//printf("setsockopt fd %d level %d optname %d poptval %x optlen %d\n",pc->sysargs[0],level,optname,poptval,optlen);
+		//printk("setsockopt fd %d level %d optname %d poptval %x optlen %d\n",pc->sysargs[0],level,optname,poptval,optlen);
 		if (optlen > 0 && poptval != umNULL) { 
+			if (__builtin_expect((optlen > MAX_SOCKOPT_LEN),0))
+				optlen=MAX_SOCKOPT_LEN;
 			optval=alloca(optlen);
 			umoven(pc,poptval,optlen,optval);
 		} else
@@ -540,7 +555,7 @@ int wrap_in_setsockopt(int sc_number,struct pcb *pc,
 }
 /* sendmsg and recvmsg have more complex arguments */
 int wrap_in_recvmsg(int sc_number,struct pcb *pc,
-		service_t sercode, sysfun um_syscall)
+		struct ht_elem *hte, sysfun um_syscall)
 {
 	int sfd=fd2sfd(pc->fds,pc->sysargs[0]);
 	if (sfd < 0) {
@@ -556,10 +571,13 @@ int wrap_in_recvmsg(int sc_number,struct pcb *pc,
 		umoven(pc,pmsg,sizeof(struct msghdr),&msg);
 		lmsg=msg;
 		if (msg.msg_namelen > 0 && msg.msg_name != NULL) {
+			if (__builtin_expect((msg.msg_namelen > MAX_SOCKET_NAME),0)) 
+				msg.msg_namelen=MAX_SOCKET_NAME;
 			lmsg.msg_name=alloca(msg.msg_namelen);
 			umoven(pc,(long)msg.msg_name,msg.msg_namelen,lmsg.msg_name);
 		}
 		if (msg.msg_iovlen > 0 && msg.msg_iov != NULL) {
+			if (__builtin_expect((msg.msg_iovlen > IOV_MAX),0)) msg.msg_iovlen=IOV_MAX;
 			iovec=alloca(msg.msg_iovlen * sizeof(struct iovec));
 			umoven(pc,(long)msg.msg_iov,msg.msg_iovlen * sizeof(struct iovec),iovec);
 		} else {
@@ -567,29 +585,34 @@ int wrap_in_recvmsg(int sc_number,struct pcb *pc,
 			msg.msg_iovlen = 0;
 		}
 		if (msg.msg_controllen > 0 && msg.msg_control != NULL) {
+			if (__builtin_expect((msg.msg_controllen > MAX_SOCK_CONTROLLEN),0))
+				msg.msg_controllen=MAX_SOCK_CONTROLLEN;
 			lmsg.msg_control=alloca(msg.msg_controllen);
 			umoven(pc,(long)msg.msg_control,msg.msg_controllen,lmsg.msg_control);
 		}
 		{
 			unsigned int i,totalsize,size;
-			char *lbuf;
+			char *lbuf, *p;
 			for (i=0,totalsize=0;i<msg.msg_iovlen;i++)
 				totalsize += iovec[i].iov_len;
-			lbuf=(char *)alloca(totalsize);
-			//fprint2("RECVMSG fd %d namesize %d msg_iovlen %d msg_controllen %d total %d\n",
+			p=lbuf=(char *)lalloca(totalsize);
+			//printk("RECVMSG fd %d namesize %d msg_iovlen %d msg_controllen %d total %d\n",
 			//		pc->sysargs[0],msg.msg_namelen, msg.msg_iovlen, msg.msg_controllen, totalsize);
 			liovec.iov_base=lbuf;
 			liovec.iov_len=totalsize;
 			lmsg.msg_iov=&liovec;
 			lmsg.msg_iovlen=1;
-			//fprint2("%d size->%d\n",sfd,size);
-			if ((size=pc->retval = um_syscall(sfd,&lmsg,flags)) < 0)
-				pc->erno=errno;
+			//printk("%d size->%d\n",sfd,size);
+			if ((pc->retval = um_syscall(sfd,&lmsg,flags)) < 0) {
+				size = 0;
+				pc->erno = errno;
+			} else
+				size = pc->retval;
 			if (size > 0) {
 				for (i=0;i<msg.msg_iovlen && size>0;i++) {
 					int qty=(size > iovec[i].iov_len)?iovec[i].iov_len:size;
-					ustoren(pc,(long)iovec[i].iov_base,qty,lbuf);
-					lbuf += qty;
+					ustoren(pc,(long)iovec[i].iov_base,qty,p);
+					p += qty;
 					size -= qty;
 				}
 			}
@@ -603,13 +626,14 @@ int wrap_in_recvmsg(int sc_number,struct pcb *pc,
 			}
 			msg.msg_flags=lmsg.msg_flags;
 			ustoren(pc,pmsg,sizeof(struct msghdr),&msg);
+			lfree(lbuf,totalsize);
 		}
 	}
 	return SC_FAKE;
 }
 
 int wrap_in_sendmsg(int sc_number,struct pcb *pc,
-		service_t sercode, sysfun um_syscall)
+		struct ht_elem *hte, sysfun um_syscall)
 {
 	int sfd=fd2sfd(pc->fds,pc->sysargs[0]);
 	if (sfd < 0) {
@@ -625,10 +649,13 @@ int wrap_in_sendmsg(int sc_number,struct pcb *pc,
 		struct iovec liovec;
 		struct iovec *iovec;
 		if (msg.msg_namelen > 0 && msg.msg_name != NULL) {
+			if (__builtin_expect((msg.msg_namelen > MAX_SOCKET_NAME),0)) 
+				msg.msg_namelen=MAX_SOCKET_NAME;
 			lmsg.msg_name=alloca(msg.msg_namelen);
 			umoven(pc,(long)msg.msg_name,msg.msg_namelen,lmsg.msg_name);
 		}
 		if (msg.msg_iovlen > 0 && msg.msg_iov != NULL) {
+			if (__builtin_expect((msg.msg_iovlen > IOV_MAX),0)) msg.msg_iovlen=IOV_MAX;
 			iovec=alloca(msg.msg_iovlen * sizeof(struct iovec));
 			umoven(pc,(long)msg.msg_iov,msg.msg_iovlen * sizeof(struct iovec),iovec);
 		} else {
@@ -636,30 +663,32 @@ int wrap_in_sendmsg(int sc_number,struct pcb *pc,
 			msg.msg_iovlen = 0;
 		}
 		if (msg.msg_controllen > 0 && msg.msg_control != NULL) {
+			if (__builtin_expect((msg.msg_controllen > MAX_SOCK_CONTROLLEN),0))
+				msg.msg_controllen=MAX_SOCK_CONTROLLEN;
 			lmsg.msg_control=alloca(msg.msg_controllen);
 			umoven(pc,(long)msg.msg_control,msg.msg_controllen,lmsg.msg_control);
 		}
 		{
-			unsigned int i,totalsize,size;
-			char *lbuf;
+			unsigned int i,totalsize;
+			char *lbuf, *p;
 			for (i=0,totalsize=0;i<msg.msg_iovlen;i++)
 				totalsize += iovec[i].iov_len;
-			lbuf=(char *)alloca(totalsize);
+			p=lbuf=(char *)lalloca(totalsize);
 			liovec.iov_base=lbuf;
 			liovec.iov_len=totalsize;
 			lmsg.msg_iov=&liovec;
 			lmsg.msg_iovlen=1;
-			char *p=lbuf;
-			//printf("SNDMSG fd %d namesize %d msg_iovlen %d msg_controllen %d total %d\n",
+			//printk("SNDMSG fd %d namesize %d msg_iovlen %d msg_controllen %d total %d\n",
 			//		pc->sysargs[0], msg.msg_namelen, msg.msg_iovlen, msg.msg_controllen, totalsize);
 			for (i=0;i<msg.msg_iovlen;i++) {
 				int qty=iovec[i].iov_len;
 				umoven(pc,(long)iovec[i].iov_base,qty,p);
 				p += qty;
 			}
-			if ((size=pc->retval = um_syscall(sfd,&lmsg,flags)) < 0)
+			if ((pc->retval = um_syscall(sfd,&lmsg,flags)) < 0)
 				pc->erno=errno;
-			//printf("%d size->%d\n",sfd,size);
+			//printk("%d size->%d\n",sfd,size);
+			lfree(lbuf,totalsize);
 		}
 	}
 	return SC_FAKE;
