@@ -18,7 +18,7 @@
  *   with this program; if not, write to the Free Software Foundation, Inc.,
  *   51 Franklin St, Fifth Floor, Boston, MA  02110-1301 USA.
  *
- *   $Id: um_wdm.c 620 2008-09-07 21:05:24Z rd235 $
+ *   $Id: um_wdm.c 866 2010-08-11 07:17:33Z rd235 $
  *
  */   
 #include <assert.h>
@@ -40,22 +40,26 @@
 #include <config.h>
 #include "defs.h"
 #include "umproc.h"
-#include "services.h"
+#include "hashtab.h"
 #include "um_services.h"
 #include "sctab.h"
 #include "scmap.h"
 #include "utils.h"
 #include "gdebug.h"
+#ifdef _VIEWOS_KM
+#include "capture_km.h"
+#endif
+
 
 int wrap_in_getcwd(int sc_number,struct pcb *pc,
-		service_t sercode, sysfun um_syscall)
+		struct ht_elem *hte, sysfun um_syscall)
 {
 	long arg1=pc->sysargs[1];
 	if (pc->sysargs[0]==(long) NULL) {
 		pc->retval=(long) NULL;
 		pc->erno=EFAULT;
 	} else {
-		//printf("dsys_getcwd %s\n",pc->fdfs->cwd);
+		//printk("dsys_getcwd %s\n",pc->fdfs->cwd);
 		int len;
 		char *root=um_getroot(pc);
 		char *cwd=pc->fdfs->cwd;
@@ -82,12 +86,34 @@ int wrap_in_getcwd(int sc_number,struct pcb *pc,
 	return SC_FAKE;
 }
 
+/* kmview: set/unset privatedir for ghost mount */
+static inline void set_wdm_kmview_chroot(struct pcb *pc)
+{
+#ifdef _VIEWOS_KM
+	/* when chroot-ed, PCB_KM_PRIVATEDIR is always set */
+	if (strcmp(pc->fdfs->root,"/")==0) {
+		/* if this is a ghost mount (returns 0 when pc->hte is NULL) */
+		if (ht_get_mountflags(pc->hte) & MS_GHOST) {
+			if ((pc->flags & PCB_KM_PRIVATEDIR) == 0) {
+				capture_km_kmpid_chroot(pc->kmpid,1);
+				pc->flags |= PCB_KM_PRIVATEDIR;
+			}
+		} else {
+			if ((pc->flags & PCB_KM_PRIVATEDIR) != 0) {
+				capture_km_kmpid_chroot(pc->kmpid,0);
+				pc->flags &= ~PCB_KM_PRIVATEDIR;
+			}
+		}
+	}
+#endif
+}
+
 /* TODO: While fchdir tries to make a chdir to the real directory instead of
  * /tmp (if it exists), chdir does not try this yet. I was not sure about
  * the correct check and where to put it, so I haven't done it at the moment.
  * But it should be changed. */
 int wrap_in_chdir(int sc_number,struct pcb *pc,
-		service_t sercode, sysfun um_syscall)
+		struct ht_elem *hte, sysfun um_syscall)
 {
 	if (!S_ISDIR(pc->pathstat.st_mode)) {
 		if (pc->pathstat.st_mode == 0)
@@ -95,22 +121,14 @@ int wrap_in_chdir(int sc_number,struct pcb *pc,
 		else
 			pc->erno=ENOTDIR;
 	}
-	if ( (pc->erno==0) && (um_x_access(pc->path,X_OK,pc)!=0) ) {
+	if ( (pc->erno==0) && (um_x_access(pc->path,X_OK,pc,&pc->pathstat)!=0) ) {
 			pc->erno=EACCES;
 	}
 	if (pc->erno == 0 && S_ISDIR(pc->pathstat.st_mode)) {
-		long sp=getsp(pc);
-		int pathlen;
-		if (sercode != UM_NONE) {
-			char *chdir_fake_dir = um_proc_fakecwd();
-			//fprint2("virtual path chdir to %s\n", chdir_fake_dir);
-			pathlen = WORDALIGN(strlen(chdir_fake_dir));
-			ustoren(pc, sp-pathlen, pathlen, chdir_fake_dir);
-		} else {
-			pathlen = WORDALIGN(strlen(pc->path));
-			ustoren(pc, sp-pathlen, pathlen, pc->path);
-		}
-		pc->sysargs[0]=sp-pathlen;
+		if (hte != NULL)
+			um_x_rewritepath(pc,um_proc_fakecwd(),0,0);
+		else
+			um_x_rewritepath(pc,pc->path,0,0);
 		return SC_CALLONXIT;
 	} else {
 		pc->retval = -1;
@@ -121,7 +139,7 @@ int wrap_in_chdir(int sc_number,struct pcb *pc,
 int wrap_out_chdir(int sc_number,struct pcb *pc) 
 {
 	if (pc->behavior == SC_FAKE) {
-		//printf("chdir err %d\n",pc->erno);
+		//printk("chdir err %d\n",pc->erno);
 		putrv(pc->retval,pc);
 		puterrno(pc->erno,pc);
 		/* we use pc->path. it has been dup-ped already */
@@ -131,63 +149,60 @@ int wrap_out_chdir(int sc_number,struct pcb *pc)
 	} else {
 		pc->retval=getrv(pc);
 		if (pc->retval >= 0) {
+			set_wdm_kmview_chroot(pc);
 			free(pc->fdfs->cwd);
 			pc->fdfs->cwd = pc->path;
 			pc->path=NULL;
-			//fprint2("new dir %d - %s\n",pc->pid, pc->fdfs->cwd);
+			//printk("new dir %d - %s\n",pc->pid, pc->fdfs->cwd);
 		} 
 		return STD_BEHAVIOR;
 	}
 }
 
 int wrap_in_fchdir(int sc_number,struct pcb *pc,
-		service_t sercode, sysfun um_syscall)
+		struct ht_elem *hte, sysfun um_syscall)
 {
-	long sp=getsp(pc);
-	int pathlen;
 	char *path;
-
-	if ((path=fd_getpath(pc->fds,pc->sysargs[0])) != NULL) {
-		//fprint2("fchdir to %s\n",path);
+	path=fd_getpath(pc->fds,pc->sysargs[0]);
+	if (path == NULL) {
+		int rv;
+		path=alloca(PATH_MAX);
+		snprintf(path,PATH_MAX,"/proc/%d/fd/%ld",pc->pid,pc->sysargs[0]);
+		if ((rv=readlink(path,path,PATH_MAX)) < 0) 
+			path=NULL;
+		else 
+			path[rv]=0;
+	}
+	if (path != NULL) {
+		//printk("fchdir to %s\n",path);
 		pc->path=strdup(path);
-		um_x_lstat64(pc->path, &(pc->pathstat), pc);
+		um_x_lstat64(pc->path, &(pc->pathstat), pc, 0);
 		/* If there is a real directory with this name, and it is chdir-able,
 		 * we can chdir there instead of /tmp/ so the core and the process
 		 * will see the same cwd. */
 		/* (rd) maybe there is a virtual dir with the same name of
 		 * a real file with X permission... 
 		 * commented out 20080626*/
-		if (S_ISDIR(pc->pathstat.st_mode) && (access(pc->path,X_OK) == 0))
+		if (S_ISDIR(pc->pathstat.st_mode) && (r_access(pc->path,X_OK) == 0))
 		{
-			pathlen = WORDALIGN(strlen(pc->path));
-			ustoren(pc, sp - pathlen, pathlen, pc->path);
-			pc->sysargs[0]=sp-pathlen;
+			um_x_rewritepath(pc,pc->path,0,0);
 			putscno(__NR_chdir, pc);
 			GDEBUG(4, "FCHDIR making fake chdir to real %s", pc->path);
 			return SC_CALLONXIT;
 		}
 		else
-#if 0
-#endif
 		{
 			if (S_ISDIR(pc->pathstat.st_mode)) {
-				if (um_x_access(pc->path,X_OK,pc)!=0) {
+				if (um_x_access(pc->path,X_OK,pc,&pc->pathstat)!=0) {
 					GDEBUG(4, "FCHDIR EACCES for %s", pc->path);
 					pc->erno=EACCES;
 					pc->retval = -1;
 					return SC_FAKE;
 				} else {
-					if (sercode != UM_NONE) {
-						char *chdir_fake_dir= um_proc_fakecwd();
-						GDEBUG(4, "FCHDIR making chdir to %s (instead of %s)", chdir_fake_dir, pc->path);
-						pathlen = WORDALIGN(strlen(chdir_fake_dir));
-						ustoren(pc, sp-pathlen, pathlen, chdir_fake_dir);
-					} else {
-						GDEBUG(4, "FCHDIR making chdir to unmanaged %s", pc->path);
-						pathlen = WORDALIGN(strlen(pc->path));
-						ustoren(pc, sp-pathlen, pathlen, pc->path);
-					}
-					pc->sysargs[0]=sp-pathlen;
+					if (hte != NULL)
+						um_x_rewritepath(pc,um_proc_fakecwd(),0,0);
+					else
+						um_x_rewritepath(pc,pc->path,0,0);
 					putscno(__NR_chdir,pc);
 					return SC_CALLONXIT;
 				}
@@ -207,7 +222,7 @@ int wrap_in_fchdir(int sc_number,struct pcb *pc,
 }
 
 int wrap_in_umask(int sc_number,struct pcb *pc,
-		service_t sercode, sysfun um_syscall)
+		struct ht_elem *hte, sysfun um_syscall)
 {
 	mode_t mode=pc->sysargs[0];
 	pc->fdfs->mask = mode;
@@ -215,23 +230,32 @@ int wrap_in_umask(int sc_number,struct pcb *pc,
 }
 
 int wrap_in_chroot(int sc_number,struct pcb *pc,
-		service_t sercode, sysfun um_syscall)
+		struct ht_elem *hte, sysfun um_syscall)
 {
-	//fprint2("CHROOT %s\n",pc->path);
+	//printk("CHROOT %s\n",pc->path);
 	if (pc->erno != 0) {
-		/* TODO management of chroot */
-		/*free(pc->fdfs->root);
-		pc->fdfs->root=strdup(pc->path) */
-		/*pc->erno = pc->retval = 0;*/
 		pc->retval = -1;
 		pc->erno = ENOENT;
 		return SC_FAKE;
 	} else {
-		if (sercode == UM_NONE) {
-			return SC_CALLONXIT;
-		} else {
-			free(pc->fdfs->root);
-			pc->fdfs->root=strdup(pc->path); 
+		/* view-OS chroot can only restrict the visible subtree,
+			 no risk for users */
+#if 0
+		if (secure && capcheck(CAP_SYS_CHROOT,pc)) {
+			pc->retval = -1;
+			pc->erno = EPERM;
+			return SC_FAKE;
+		} else 
+#endif
+		{
+			if (strcmp(pc->fdfs->root,pc->path) != 0) {
+#ifdef _VIEWOS_KM
+				if (strcmp(pc->fdfs->root,"/")==0)
+					capture_km_kmpid_chroot(pc->kmpid,1);
+#endif
+				free(pc->fdfs->root);
+				pc->fdfs->root=strdup(pc->path); 
+			}
 			pc->retval = 0;
 			pc->erno = 0;
 			return SC_FAKE;
