@@ -1,3 +1,29 @@
+/*   This is part of um-ViewOS
+ *   The user-mode implementation of OSVIEW -- A Process with a View
+ *
+ *   umbinwrap.c: wrapper for executables. In ViewOS "execve" executes 
+ *   this program.
+ *   If mmap does not work for the executable (it is the case of some
+ *   virtual file systems) it copies the executable and uses a watchdog
+ *   process to clean up the copy when the execution terminates.
+ * 
+ *   Copyright 2007 Renzo Davoli University of Bologna - Italy
+ *   
+ *   This program is free software; you can redistribute it and/or modify
+ *   it under the terms of the GNU General Public License, version 2, as
+ *   published by the Free Software Foundation.
+ *
+ *   This program is distributed in the hope that it will be useful,
+ *   but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *   GNU General Public License for more details.
+ *
+ *   You should have received a copy of the GNU General Public License along
+ *   with this program; if not, write to the Free Software Foundation, Inc.,
+ *   51 Franklin St, Fifth Floor, Boston, MA  02110-1301 USA.
+ *
+ */
+
 #include <stdio.h>
 #include <string.h>
 #include <alloca.h>
@@ -6,6 +32,8 @@
 #include <errno.h>
 #include <stdlib.h>
 #include <sys/mman.h>
+#include <sys/types.h>
+#include <sys/stat.h>
 #include <config.h>
 
 /* return value: 0=the file is mmapable, 1 otherwise */
@@ -75,7 +103,7 @@ int main(int argc,char* argv[])
 	char *s=argv[0];
 	char sep=*s;
 	char *cmd;
-	char **newargv=alloca(argc+3);
+	char **newargv=alloca((argc+3)*sizeof(char *));
 	int i;
 	int sargc=0;
 	*s=0;
@@ -100,7 +128,7 @@ int main(int argc,char* argv[])
 	newargv[i+sargc]=0;
 #if 0
 	for (i=0;i<argc+sargc;i++)
-		printf("%d %s\n",i,newargv[i]);
+		fprintf(stderr,"%d %s\n",i,newargv[i]);
 #endif
 	if (mmap_not_ok(newargv[1])) 
 		execv_nommap(cmd,newargv);
