@@ -19,7 +19,7 @@
  *   with this program; if not, write to the Free Software Foundation, Inc.,
  *   51 Franklin St, Fifth Floor, Boston, MA  02110-1301 USA.
  *
- *   $Id: umproc.h 582 2008-06-26 09:50:31Z rd235 $
+ *   $Id: umproc.h 886 2010-08-23 15:48:38Z rd235 $
  *
  */   
 #ifndef _UMPROC_H
@@ -28,7 +28,7 @@
 #define OLFD_STEP 8 /*only power of 2 values */
 #define OLFD_STEP_1 (OLFD_STEP - 1)
 
-#include "services.h"
+#include "hashtab.h"
 #include "treepoch.h"
 
 #ifdef _UM_MMAP
@@ -36,6 +36,14 @@ extern int um_mmap_secret;
 extern int um_mmap_pageshift;
 #endif
 
+#ifdef _UM_EPOLL
+#include <sys/epoll.h>
+struct lfd_epoll_item {
+	int fd;
+	struct epoll_event event;
+};
+#endif
+
 /* informations about the full set of file descriptors of a process */
 struct pcb_file {
 	/* with CLONE_FILES, more than one process can refer to the same file
@@ -57,19 +65,20 @@ char *um_proc_tmpname();
 void lfd_addproc (struct pcb_file **p,int flag);
 void lfd_delproc (struct pcb_file *p);
 #endif
-int lfd_open (service_t service, int sfd, char *path, int flags, int nested);
+int lfd_open (struct ht_elem *hte, int sfd, char *path, int flags, int nested);
 void lfd_close (int lfd);
 int lfd_dup(int lfd);
 int lfd_getcount(int lfd);
 void lfd_nullsfd(int lfd);
 int lfd_getsfd(int lfd);
-service_t lfd_getservice(int lfd);
+struct ht_elem *lfd_getht(int lfd);
 char *lfd_getfilename(int lfd);
 char *lfd_getpath(int lfd);
 int fd2lfd (struct pcb_file *p, int fd);
 int fd_getfdfl(struct pcb_file *p, int fd);
 int fd_setfdfl(struct pcb_file *p, int fd, int val);
 int fd_getflfl(struct pcb_file *p, int fd);
+int fd_setflfl(struct pcb_file *p, int fd, int flags);
 int fd2sfd (struct pcb_file *p, int fd);
 char *fd_getpath(struct pcb_file *p, int fd);
 void lfd_register (struct pcb_file *p, int fd, int lfd);
@@ -77,7 +86,17 @@ void lfd_deregister_n_close(struct pcb_file *p, int fd);
 void lfd_closeall();
 void lfd_signal(int lfd);
 void lfd_delsignal(int lfd);
-service_t service_fd(struct pcb_file *p, int fd, int setepoch);
-char *sfd_getpath(service_t code, int sfd);
+struct ht_elem *ht_fd(struct pcb_file *p, int fd, int setepoch);
+char *sfd_getpath(struct ht_elem *hte, int sfd);
+
+#ifdef _UM_EPOLL
+void lfd_epoll_add(struct pcb_file *p,int epfd,struct lfd_epoll_item *ep_item);
+//void lfd_epoll_mod(struct pcb_file *p,int epfd,struct lfd_epoll_item *ep_item);
+void lfd_epoll_del(struct pcb_file *p,int epfd,int fd);
+struct lfd_epoll_item *lfd_epoll_search(struct pcb_file *p,int epfd,int fd);
+void lfd_epoll_forall(struct pcb_file *p, int epfd,
+		void (* fun)(struct lfd_epoll_item *epoll,void *arg),
+		void *arg);
 
 #endif
+#endif
