@@ -19,7 +19,7 @@
  *   with this program; if not, write to the Free Software Foundation, Inc.,
  *   51 Franklin St, Fifth Floor, Boston, MA  02110-1301 USA.
  *
- *   $Id: umview.c 668 2009-05-30 15:08:31Z rd235 $
+ *   $Id: umview.c 974 2011-08-08 08:52:20Z rd235 $
  *
  */   
 #include <stdio.h>
@@ -28,13 +28,16 @@
 #include <dlfcn.h>
 #include <fcntl.h>
 #include <poll.h>
+#include <errno.h>
 #include <sys/select.h>
 #include <sys/time.h>
 #include <sys/resource.h>
 #include <stdlib.h>
 #include <assert.h>
 #include <signal.h>
+#ifdef OLDVIRSC
 #include <linux/sysctl.h>
+#endif
 #include <config.h>
 #include <loginshell.h>
 
@@ -54,9 +57,9 @@
 #include "gdebug.h"
 
 #ifdef GDEBUG_ENABLED
-#	define OPTSTRING "+p:f:o:hvnxqV:"
+#	define OPTSTRING "+p:f:o:hvnxqV:s"
 #else
-#	define OPTSTRING "+p:f:hvnxqV:"
+#	define OPTSTRING "+p:f:hvnxqV:s"
 #endif
 
 int _umview_version = 2; /* modules interface version id.
@@ -65,12 +68,11 @@ int _umview_version = 2; /* modules interface version id.
 unsigned int has_ptrace_multi;
 unsigned int ptrace_vm_mask;
 unsigned int ptrace_sysvm_tag;
-unsigned int hasppoll;
 unsigned int quiet = 0;
+unsigned int printk_current_level = PRINTK_STARTUP_LEVEL;
+unsigned int secure = 0;
 static char *viewname;
 
-unsigned int want_ptrace_multi, want_ptrace_vm, want_ptrace_viewos, want_ppoll;
-
 extern int nprocs;
 
 struct prelist {
@@ -92,6 +94,7 @@ static void preadd(struct prelist **head,char *module)
 }
 
 /* virtual syscall for the underlying umview */
+#ifdef OLDVIRSC
 static long int_virnsyscall(long virscno,int n,long arg1,long arg2,long arg3,long arg4,long arg5,long arg6) {
 	struct __sysctl_args scarg;
 	long args[6]={arg1,arg2,arg3,arg4,arg5,arg6};
@@ -103,21 +106,23 @@ static long int_virnsyscall(long virscno,int n,long arg1,long arg2,long arg3,lon
 	scarg.newlen=n;
 	return native_syscall(__NR__sysctl,&scarg);
 }
+#else
+static long int_virnsyscall(long virscno,int n,long arg1,long arg2,long arg3,long arg4,long arg5,long arg6) {
+	long args[6]={arg1,arg2,arg3,arg4,arg5,arg6};
+	return native_syscall(__NR_pivot_root,NULL,n,virscno,args);
+}
+#endif
 
 /* preload of modules */
 static int do_preload(struct prelist *head)
 {
 	if (head != NULL) {
-		void *handle;
 		int rv=do_preload(head->next);
-		handle=open_dllib(head->module);
-		if (handle==NULL) {
-			fprintf(stderr, "%s\n",dlerror());
-			return -1;
-		} else {
-			set_handle_new_service(handle,0);
+		if (add_service(head->module,0) < 0) {
+			printk("module preload %s",strerror(errno));
+			return -1 ;
+		} else
 			return rv;
-		}
 		free(head);
 	} else
 		return 0;
@@ -136,7 +141,7 @@ static int do_preload_recursive(struct prelist *head)
 {
 	if (head != NULL) {
 		do_preload_recursive(head->next);
-		int_virnsyscall(__NR_UM_SERVICE,3,ADD_SERVICE,0,(long)head->module,0,0,0);
+		int_virnsyscall(__NR_UM_SERVICE,3,ADD_SERVICE,(long)head->module,0,0,0,0);
 		free(head);
 		return 0;
 	} else
@@ -146,7 +151,7 @@ static int do_preload_recursive(struct prelist *head)
 static void do_set_viewname_recursive(char *viewname)
 {
 	if (viewname) {
-		int_virnsyscall(__NR_UM_SERVICE,2,UMVIEW_SETVIEWNAME,(long)viewname,0,0,0,0);
+		int_virnsyscall(__NR_UM_SERVICE,2,VIEWOS_SETVIEWNAME,(long)viewname,0,0,0,0);
 	}
 }
 
@@ -185,7 +190,7 @@ static void usage(char *s)
 			"  --nokmulti                avoid using PTRACE_MULTI\n"
 			"  --noksysvm                avoid using PTRACE_SYSVM\n"
 			"  --nokviewos               avoid using PTRACE_VIEWOS\n\n"
-			"  --noppoll                 avoid using ppoll\n\n",
+			"  -s, --secure              force permissions and capabilities\n",
 			s);
 	exit(0);
 }
@@ -205,7 +210,7 @@ static struct option long_options[] = {
 	{"nokmulti",0,0,0x100},
 	{"noksysvm",0,0,0x101},
 	{"nokviewos",0,0,0x102},
-	{"noppoll",0,0,0x103},
+	{"secure",0,0,'s'},
 	{0,0,0,0}
 };
 
@@ -213,6 +218,7 @@ static struct option long_options[] = {
 static void load_it_again(int argc,char *argv[],int login)
 {
 	int nesting=1;
+	optind=1;
 	while (1) {
 		int c;
 		int option_index = 0;
@@ -306,24 +312,14 @@ static void umview_recursive(int argc,char *argv[])
 	exit(-1);
 }
 
-/*
-static int has_pselect_test()
-{
-#ifdef _USE_PSELECT
-	static struct timespec to={0,0};
-	return (r_pselect6(0,NULL,NULL,NULL,&to,NULL)<0)?0:1;
-#else
-	return 0;
-#endif
-}
-*/
-
 #include<errno.h>
 /* UMVIEW MAIN PROGRAM */
 int main(int argc,char *argv[])
 {
 	char *rcfile=NULL;
-	if (argc == 1 && argv[0][0] == '-') /* login shell */
+	unsigned int want_ptrace_multi, want_ptrace_vm, want_ptrace_viewos;
+	sigset_t unblockchild;
+	if (argc == 1 && argv[0][0] == '-' && argv[0][1] != '-') /* login shell */
 		loginshell_view();
 	/* try to set the priority to -11 provided umview has been installed
 	 * setuid. it is effectiveless elsewhere */
@@ -331,17 +327,19 @@ int main(int argc,char *argv[])
 	/* if it was setuid, return back to the user status immediately,
 	 * for safety! */
 	r_setuid(getuid());
+	/* Check these cases only when *not* reloaded for purelibc */
+	if (strncmp(argv[0],"--umview",8)!=0) {
 	/* if this is a nested invocation of umview, notify the umview monitor
 	 * and execute the process, 
 	 * try the nested invocation notifying virtual syscall, 
 	 * if it succeeded it is actually a nested invocation,
 	 * otherwise nobody is notified and the call fails*/
-	if (int_virnsyscall(__NR_UM_SERVICE,1,RECURSIVE_UMVIEW,0,0,0,0,0) >= 0)
-		umview_recursive(argc,argv);	/* do not return!*/
-	/* umview loads itself twice if there is pure_libc, to trace module 
-	 * generated syscalls, this condition manages the first call */
-	if (strncmp(argv[0],"--umview",8)!=0)
+		if (int_virnsyscall(__NR_UM_SERVICE,1,RECURSIVE_VIEWOS,0,0,0,0,0) >= 0)
+			umview_recursive(argc,argv);	/* do not return!*/
+		/* umview loads itself twice if there is pure_libc, to trace module 
+		 * generated syscalls, this condition manages the first call */
 		load_it_again(argc,argv,isloginshell(argv[0]));	/* do not return (when purelibc and not -x)!*/
+	}
 
 	if (argc < 2)
 	{
@@ -357,10 +355,8 @@ int main(int argc,char *argv[])
 	scdtab_init();
 	/* test the ptrace support */
 	has_ptrace_multi=test_ptracemulti(&ptrace_vm_mask,&ptrace_sysvm_tag);
-	hasppoll=hasppolltest();
 	want_ptrace_multi = has_ptrace_multi;
 	want_ptrace_vm = ptrace_vm_mask;
-	want_ppoll = hasppoll;
 	/* option management */
 	while (1) {
 		int c;
@@ -403,6 +399,9 @@ int main(int argc,char *argv[])
 					 want_ptrace_vm = 0;
 					 want_ptrace_viewos = 0;
 					 break;
+			case 's':
+					 secure=1;
+					 break;
 			case 0x100: /* do not use ptrace_multi */
 					 want_ptrace_multi = 0;
 					 break;
@@ -412,27 +411,22 @@ int main(int argc,char *argv[])
 			case 0x102: /* do not use ptrace_viewos */
 					 want_ptrace_viewos = 0;
 					 break;
-			case 0x103: /* do not use ppoll */
-					 want_ppoll = 0;
-					 break;
 		}
 	}
 	
 	if (!quiet)
 	{
-		if (has_ptrace_multi || ptrace_vm_mask || hasppoll)
+		if (has_ptrace_multi || ptrace_vm_mask)
 		{
 			fprintf(stderr, "This kernel supports: ");
 			if (has_ptrace_multi)
 				fprintf(stderr, "PTRACE_MULTI ");
 			if (ptrace_vm_mask)
 				fprintf(stderr, "PTRACE_SYSVM ");
-			if (hasppoll)
-				fprintf(stderr, "ppoll ");
 			fprintf(stderr, "\n");
 		}
 
-		if (has_ptrace_multi || ptrace_vm_mask || hasppoll ||
+		if (has_ptrace_multi || ptrace_vm_mask ||
 				want_ptrace_multi || want_ptrace_vm || want_ptrace_viewos)
 		{
 			fprintf(stderr, "%s will use: ", UMVIEW_NAME);	
@@ -442,9 +436,7 @@ int main(int argc,char *argv[])
 				fprintf(stderr,"PTRACE_SYSVM ");
 			if (want_ptrace_viewos)
 				fprintf(stderr,"PTRACE_VIEWOS ");
-			if (want_ppoll)
-				fprintf(stderr,"ppoll ");
-			if (!want_ptrace_multi && !want_ptrace_vm && !want_ptrace_viewos && !want_ppoll)
+			if (!want_ptrace_multi && !want_ptrace_vm && !want_ptrace_viewos)
 				fprintf(stderr,"nothing");
 			fprintf(stderr,"\n\n");
 		}
@@ -452,40 +444,26 @@ int main(int argc,char *argv[])
 
 	has_ptrace_multi = want_ptrace_multi;
 	ptrace_vm_mask = want_ptrace_vm;
-	hasppoll = want_ppoll;
 	
 	if (rcfile==NULL && !isloginshell(argv[0]))
 		asprintf(&rcfile,"%s/%s",getenv("HOME"),".viewosrc");
 
-	if (hasppoll) {
-		sigset_t unblockchild;
-		sigprocmask(SIG_BLOCK,NULL,&unblockchild);
-		pcb_inits(1);
-		capture_main(argv+optind,1,rcfile);
-		setenv("_INSIDE_UMVIEW_MODULE","",1);
-		do_preload(prehead);
-		do_set_viewname(viewname);
-		while (nprocs) {
-			mp_ppoll(&unblockchild);
-			tracehand();
-		}
+	if (quiet) {
+		setenv("_VIEWOS_QUIET","1",1);
+		printk_current_level = PRINTK_QUIET_LEVEL; /* warnings or errors only */
 	}
-	else {
-		int wt=wake_tracer_init();
-		/* logically should follow pcb_inits, but the wake tracer fd will be the
-		 * most hit, so this inversion gives performance to the system */
-		mp_add(wt,POLLIN,do_wake_tracer,NULL,1);
-		pcb_inits(0);
-		capture_main(argv+optind,0,NULL);
-		setenv("_INSIDE_UMVIEW_MODULE","",1);
-		do_preload(prehead);
-		do_set_viewname(viewname);
-		while (nprocs)  {
-			mp_poll();
-			do_wake_tracer();
-		}
+
+	sigprocmask(SIG_BLOCK,NULL,&unblockchild);
+	pcb_inits(1);
+	capture_main(argv+optind,rcfile);
+	setenv("_INSIDE_VIEWOS_MODULE","",1);
+	do_preload(prehead);
+	do_set_viewname(viewname);
+	while (nprocs) {
+		mp_ppoll(&unblockchild);
+		tracehand();
 	}
-	pcb_finis(hasppoll);
+	pcb_finis(1);
 	return first_child_exit_status;
 }
 
