@@ -18,14 +18,14 @@
  *   with this program; if not, write to the Free Software Foundation, Inc.,
  *   51 Franklin St, Fifth Floor, Boston, MA  02110-1301 USA.
  *
- *   $Id: umview.h 570 2008-06-18 08:05:16Z garden $
+ *   $Id: umview.h 668 2009-05-30 15:08:31Z rd235 $
  *
  */
 
 #include <config.h>
 
 #define UMVIEW_NAME PACKAGE_NAME
-#define UMVIEW_DESC "The user-mode implementation of View-OS"
+#define UMVIEW_DESC "umview: The user-mode implementation of View-OS"
 #define UMVIEW_VERSION PACKAGE_VERSION
 
 #define UMVIEW_URL "http://view-os.sourceforge.net"
